/* BesselHorn script by Brian McMillin, June, 2022.
The BesselHorn script builds a trombone-like flared bell profile curve.
References:
http://www.acoustics.ed.ac.uk/wp-content/uploads/Theses/Braden_Alistair__PhDThesis_UniversityOfEdinburgh_2006.pdf

Appendix B of the Thesis describes the orientation of the profile curve.
"The horn lies to the left of the origin, with the mouth facing right."
The Thesis has the equation for the Bessel Horn profile curve,
as well as the two equations for calculating the fitting parameter, bValue,
and the value of x0, (far left), for the throat radius rValue at the left.

The following link uses the thesis equations.
https://github.com/pieterbos/PrintBone

There are four parameters.
1.  Throat Radius = rValue, is the beginning (smaller) radius associated with x0 at the left side.
(x values are negative.)
2.  Mouth Radius, RValue, is the ending (larger) radius at the right side, near the y-axis.
3.  Flair, fValue, is the constant which governs the rate of increase of the bell radius.
4.  Length, dValue, is the length of the horn, along the symmetry axis down the center of the horn.

bValue and x0 are calculated from the previous 4 parameters.

The flair slider, fslider, will vary the shape of the profile curve for fixed rValue, RValue, and dValue.

Centimeters can be the default unit of distance.

The Build Cone Curve selection just makes a line.

Selecting Exponential Curve builds an exponential horn curve.
Note that flare = (2/dValue) * ln(R1/R0), so there is only one exponential horn curve between two points.  (Again, see Appendix B of thesis.)
Curve forms from x = 0.0 to x = length.
Adding negative x0 shifts curve to left, aligning x values with bessel curve.

Instead, Bessel and Cone curves are shifted to right of origin and y-axis, by code,
and Exponential stays at +x values.)

*/

function EvalBesselHornCurve(tdx, bValue, fValue, x0, bottom)
{
	var z = 0.0;
	var x = tdx - x0; // To shift curve right of y-axis;
	var y = bValue * Math.pow( -tdx, -fValue );
	if (bottom)
		y *= -1.0;
	return moi.vectorMath.createPoint( x, y, z );
}

function EvalExponentialHornCurve(tdx, rValue, eFlare, x0, bottom)
{
	var z = 0.0;	
	var y = rValue * Math.exp( eFlare * tdx / 2 );
	// tdx += x0; // Would shift curve left;
	if (bottom)
		y *= -1.0;
	return moi.vectorMath.createPoint( tdx, y, z );
}

function Update()
{
	var UI = moi.ui.commandUI;
	var rValue = UI.rValue.numericValue;
	var RValue = UI.RValue.numericValue;
	var fValue = UI.fValue.numericValue;
	var dValue = UI.dValue.numericValue;
	var numpoints = UI.numpoints.integerValue;
	var buildtype = UI.Build.value;
	var bottom = UI.bottomcurve.value;	

//	moi.ui.alert( 'Throat radius = ' + rValue + ' cm' + '\n' + 'Mouth radius = ' + RValue + ' cm' + '\n' + 'Flair = ' + fValue + '\n' + 'Length = ' + dValue + ' cm' );

	var bValue = Math.pow(( dValue/(Math.pow(rValue, -1/fValue) - Math.pow(RValue, -1/fValue))), fValue);
//	moi.ui.alert( 'bValue = ' + bValue );

	var x0 = -Math.pow((rValue/bValue), (-1/fValue))
//	moi.ui.alert( 'x0 = ' + x0 + ' cm' );

// How to get "x" values to calculate "y" radii:
// i ranges from 0 to (numpoints-1).
// n = (numpoints -1)
// t = i/n
// t is element of: (0, 1/n, 2/n, ..., n/n)
// t*dValue is element of: (0, ..., dValue)
// Now shift the range by the value of x0.  (Shifts to left if x0 is negative.)
// t*dValue + x0 = (x0, ..., dValue+x0)
// (Note that x0 is negative, and dValue, (distance), is positive.)

  if ( buildtype == 'Bessel' )
  {
	var factory = moi.command.createFactory( 'interpcurve' );	  
	for ( var i = 0; i < numpoints; ++i )
	{
		var t = i / (numpoints - 1);
		var tdx = t*dValue + x0;
		var pt = EvalBesselHornCurve(tdx, bValue, fValue, x0, bottom);
		factory.createInput( 'point' );
		factory.setInput( factory.numInputs - 1, pt );	
	}
	factory.update();
	return factory;
  }
  else if ( buildtype == 'Conical' )
  {
	// Build cone line.		
	var factory = moi.command.createFactory( 'line' );

	if (bottom)
	{
	rValue *= -1.0;
	RValue *= -1.0;
	}			
			
	// var x1 = dValue + x0;  //For cone curve.
	var x1 = dValue;  // Keep curve to right of y-axis.	
//	moi.ui.alert( 'x1 = ' + x1 + ' cm' + '\n' + 'dValue = ' + dValue + ' cm' + '\n' + 'x0 = ' + x0 + ' cm' );
	var z = 0.0;
	x = 0.0; // Begin curve at y-axis.
	var pt0 = moi.vectorMath.createPoint( x, rValue, z );
	var pt1 = moi.vectorMath.createPoint( x1, RValue, z );		
	var factory = moi.command.createFactory( 'line' );
	factory.setInput( 0, pt0 );
	factory.setInput( 1, pt1 );
	factory.update();
	return factory;		
  }
  else
  {
	// Build exponential curve.		
	var factory = moi.command.createFactory( 'interpcurve' );	  
	for ( var i = 0; i < numpoints; ++i )
	{
		var t = i / (numpoints - 1);


//		var tdx = t*dValue + x0;
		var tdx = t*dValue;  // x0 to be added later.
		eFlare = (2/dValue) * Math.log(RValue/rValue);
		
		var pt = EvalExponentialHornCurve(tdx, rValue, eFlare, x0, bottom);
		factory.createInput( 'point' );
		factory.setInput( factory.numInputs - 1, pt );	
  }
	factory.update();
	return factory;
  }
}

function DoBesselHorn()
{
	var factory = Update() 
	
	var dlg = moi.ui.commandDialog;
	
	while ( 1 )
	{
		if ( !dlg.waitForEvent() )
		{
			factory.cancel();
			return false;
		}
			
		if ( dlg.event == 'done' )
			break;
			
		if ( dlg.event == 'rValue' || dlg.event == 'RValue' || dlg.event == 'fValue' || dlg.event == 'dValue' || dlg.event == 'numpoints' || dlg.event == 'rslider'  || dlg.event == 'Rslider' || dlg.event == 'fslider'  || dlg.event == 'dslider' || dlg.event == 'Build' || dlg.event == 'bottomcurve' )
		{
			factory.cancel();
			factory = Update();			
			moi.ui.commandUI.updating = false; //for slider update.		
		}		
	}
	factory.commit();
}
DoBesselHorn();







