// config: norepeat

// This script _BraidTrack, creates curves (strand centerlines) from points on a racetrack curve.
// The racetrack curve could be 3D and closed or unclosed,
// but for this description, the racetrack is considered to be a 2D curve in the Front view.
// The racetrack curve could be a circle, ellipse, polygon, "figure 8,"
// multilobed figure 8, or Lissajous curve for a Braid.
// The starting points of the curves are positioned as points on the racetrack curve.
// Each point may represent the start point of the centerline of a Braid strand.
// The points are imagined to be travelling along the curve in time, at constant speed.
// The points are arrayed along the racetrack with perhaps 200 copies, each position a different Time.
// The y-coordinates of the arrayed points are modified by adding fractions of Time, ( fractions of y distance.) 
// The 3D positions of the centerline points are plotted with their y-coordinates as Time.
// Each point has two possible directions of travel.  Array seems to work well in selecting the direction.
// Running FLIP changes the direction of travel, with the same result as a mirror.
// For a "figure 8" racetrack, on one lobe travel is clockwise, and ccw on the other lobe.
// For a 3 lobe racetrack, the two outer lobes are cw, and the inner lobe ccw, or vice versa.
// A previous Braid script, BraidLF02, used Lissajous curves, in parametric form.
// The racetrack option showed the Lissajous racetrack, and starting points.
// Direction of travel along the lobes was discernible as a combination of up, down, left, or right.
// This script may use NURBS curves with similar lobes and start points, but with smoother curvature,
// to make a more realistic braid.
// Access to the NURBS parametric equations might permit the points to be plotted by equation,
// like in a (non-existent) factory.
// Other interpretations are possible.
// Whether such racetracks are useful remains to be determined.
// If a 2D racetrack were extruded, the curves would be on the surface. 
// This script has code modified from LineWeb script, the new BlendCap script, and ConnectPoints script. 
// By Brian McMillin, June, 2015.

#include "GetObjects.js"

function Cancel( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].cancel();
		
	factories.length = 0;
}

function Commit( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].commit();
}

// curve is the racetrack.
// pts are the starting points on the racetrack, for each strand.
// Each starting point is arrayed on the racetrack, and a y-axis offset is added.
function Update( curve, pts, numpoints, yScale, factories )
{
	Cancel( factories );
		
// There is only one racetrack curve object in curve(s).

	var pt_factory = moi.command.createFactory( 'point' );
	
	// Run the for loop for each strand start point.	
	for ( var i = 0; i < pts.length; ++i )
	{
		var pti = pts.item(i);// start point
		
//	In this script, pti is a objectpicker picked point, NOT an object list,
//	so convert it to the proper object list,
//	and Array it along the curve to produce a list of point objects.
//	Remark, In other scripts, like LineWeb, in order to create a point_object_list
//	to array, the following 3 lines of code are used:
//		var pointfactory = moi.command.createFactory( 'point' );
//		pointfactory.setInput( 0, curve.getStartPt() );
//		var start point = pointfactory.calculate(); //This is an object list.

		// In this script, a point object to array is created with the following 2 lines:
		pt_factory.setInput( 0, pti.pt );
		var ptiPtObject = pt_factory.calculate(); //An object list

//	Remark, another way to convert pti into an object list, as per ChainMaker script,
//	section /*Make object list with link1 and link2 (Michaels help used)*/
//	As follows:		
//		var ptiPtObject = moi.geometryDatabase.createObjectList();
//		ptiPtObject.addObject( pti );

		// Array point along the curve to produce a list of point objects.
		var arrayfactory = moi.command.createFactory( 'arraycurve' );
		arrayfactory.setInput( 0, ptiPtObject );	 // Object list	
		arrayfactory.setInput( 1, curve );			// Path
		arrayfactory.setInput( 2, 'NumItems' ); 	// Mode
		arrayfactory.setInput( 3, numpoints );  	// Num items
		var arrayed_pts = arrayfactory.calculate();// An object list.

//	The following two lines are a Test display of the points on the racetrack, and should be removed.
		//if ( arrayed_pts )
		//moi.geometryDatabase.addObjects( arrayed_pts );	
		
		// var arrayed_pts contains all the newly created points on the racetrack curve,
		// except for the start point ptiPtObject, and also, if the racetrack curve is closed,
		// it does not include the end point, ptiPtObject, which is the same point as the start point.
		// The end point will have its y-coordinate modified.
 
		var factory = moi.command.createFactory( 'interpcurve' );
		factory.createInput( 'point' );
		factory.setInput( 0, pti.pt ); //start point
		
		for ( var j = 0; j < arrayed_pts.length; ++j )
		{
			var t = (j+1) / ( arrayed_pts.length );// if curve is not closed, max t = length/length = 1.
			if  (curve.isClosed)
				t = (j+1) / ( arrayed_pts.length + 1 );// max t = length/(length+1)	
			var pRaw = arrayed_pts.item(j).pt;
//			pRaw.y = pRaw.y + 2 * Math.PI * t;			
			pRaw.y = pRaw.y + yScale * t;
			var pMod = moi.vectorMath.createPoint( pRaw.x, pRaw.y, pRaw.z );
			factory.createInput( 'point' );
			factory.setInput( (j+1), pMod );
		}
		if  (curve.isClosed)
		{
			var endptRaw = pti.pt;
//			endptRaw.y = endptRaw.y + 2 * Math.PI;			
			endptRaw.y = endptRaw.y + yScale;
			var endptMod = moi.vectorMath.createPoint( endptRaw.x, endptRaw.y, endptRaw.z );
			factory.createInput('point');
			factory.setInput(factory.numInputs - 1, endptMod);			
		}
		
		factories.push( factory );		
	}	

	for ( var i = 0; i < factories.length; ++i )
		factories[i].update();
}

function BraidTrack()
{
	var ui = moi.ui;
	var vm = moi.vectorMath;
	var gd = moi.geometryDatabase;
	
//	Select the racetrack.
	var objectpicker = ui.createObjectPicker();
	objectpicker.allowStandaloneCurves();
	if ( !GetObjects( objectpicker ) )
		return;

	var curves = objectpicker.objects;
	curves.sortBySelectionOrder();
	curves.setProperty( 'locked', true );
	var curve = curves.item(0);// One racetrack
	
	moi.ui.beginUIUpdate();
	moi.ui.hideUI( 'SelectPrompt1' );
	moi.ui.showUI( 'SelectPrompt2' );
	moi.ui.endUIUpdate();
	
//  Select points on racetrack corresponding to Strand start points.
//	These points should be placed similarly to those points on a Lissajous racetrack.
	var objectpicker = ui.createObjectPicker();
	objectpicker.allowPointObjects();
	if ( !GetObjects( objectpicker ) )
		return;

	var pts = objectpicker.objects;//Strand start points	
//	pts.sortBySelectionOrder();	

	ui.beginUIUpdate();
	ui.hideUI( 'SelectPrompt2' );
	ui.showUI( 'OptionsPrompt' );
	ui.showUI( 'Options' );
	ui.endUIUpdate();

	var factories = new Array();
	var UI = moi.ui.commandUI;	
	
	Update( curve, pts, UI.numpoints.integerValue, UI.yScale.numericValue, factories );

	// Wait for cancel or done.
	var dialog = moi.ui.commandDialog;
	while ( 1 )
	{
		if ( !dialog.waitForEvent() )
		{
			Cancel( factories );
			return;
		}

		if ( dialog.event == 'done' )
			break;

		if ( dialog.event == 'numpoints' || dialog.event == 'yScale' )
			Update( curve, pts, UI.numpoints.integerValue, UI.yScale.numericValue, factories );
	}
	
	Commit( factories );
}

BraidTrack();
