//	Catenary2PtXY is a script which produces a catenary in the xy plane,
//	from two points picked by the user.
//	Gravity is in the negative y direction.
//	In the event that the z coordinates of the two points are not equal,
//	an alert requiring equal z coordinates is given.
//	This script uses the Opalg math, and most of the elbarto code.
//	The MoI Distance script is used to select and measure the distance
//	between the two points.  The user inputs the chain length, which must be
//	greater than the Distance, or a catenary will not form.
//	In the event that a catenary does not form, even though the chain length
//	is greater than the Distance, OR the catenary is upside down, 
//	Newton-Raphson has failed, and the user can adjust the GuessRadius
//	parameter, to be smaller or greater than 1.0.
//	For a chain length that is long in relation to the x-axis
//	distance, try a fractional GuessRadius. 
//	See PDF "Catenary Suspension from Two Points."

//	An ARCH seems to be formed if a negative GuessRadius is used.

#include "GetPoint.js"

function alert( msg )
{
	moi.ui.commandUI.alert( msg );
}

//	Define cosh and sinh.
function cosh(x)
{
	return (Math.exp(x) + Math.exp(-x))/2.0
}

function sinh(x)
{
	return (Math.exp(x) - Math.exp(-x))/2.0
}

//	Find aParameter with the Newton-Raphson Method of solving a Nonlinear Equation.
function evalNewtRap( aGuess, d, sqrtSShh, tol, maxiter )
{
	var cur_a = aGuess
	for ( var j = 0; j < maxiter; ++j )
	{
		var f_a = 2.0 * cur_a * sinh(d/(2.0*cur_a)) - sqrtSShh;
 		var Df_a = 2.0 * sinh(d/(2.0 * cur_a)) - d * cosh(d/(2.0 * cur_a)) /cur_a ;

		cur_a -= (f_a/Df_a);

		if (Math.abs(f_a) < tol)
		{
			return cur_a;
		}
	}		
}

//	Update the catenary.
function Update(BasePt, OtherPt)
{
	var ui = moi.ui;
	var vm = moi.vectorMath;	//shorthand.
	var catfactory = moi.command.createFactory( 'curve' );

//	"d" is the horizontal x distance between P1 and P2.
//	"h" is the vertical y distance between P1 and P2.
//	P1 = (P1.x, P1.y, P1.z)
//	P2 = (P2.x, P2.y, P2.z)


//	The Point with the smaller x value is to be P1.
	if ( BasePt.x < OtherPt.x )
	{
		var P1 = BasePt;
		var P2 = OtherPt;
	}
	else
	{
		var P1 = OtherPt;
		var P2 = BasePt;	
	}

	
//	From above if/else, d must be positive.
//	h could be positive or negative,and needs to be so.

	var d = (P2.x - P1.x);
	var h = (P2.y - P1.y);	

//	S is the arclength of the catenary, and must be greater than Distance between P1 & P2.
//	S is the length of the chain, and the value of S must be provided by the user.
	var S = ui.commandUI.S.numericValue;

//	"aParam" is the radius of curvature at the vertex of the catenary, and needs to be
//	solved for, using a numeric method.  The Newton-Raphson method is used, and requires
//	an initial guess for aParam, called "aGuess", initially set to 1.0 units.  If the
//	catenary fails to apear, and S > Distance, then adjusting aGuess may help.	
	var aGuess = ui.commandUI.aGuess.numericValue;

//	"numpoints" is the number of catenary points calculated, in order to plot the catenary.
//	The catfactory draws the catenary curve using control points, so the curve will be
//	very close to the control points, but may not be exactly on the points. 
	var numpoints = ui.commandUI.numpoints.integerValue;
	
//	tolerance and maxiterations are used in the Newton-Raphson method, and can be adjusted.	
	var tol = ui.commandUI.tol.numericValue;
	tol /= 10000.0;	  //Divide tol by 10,000 for input field display. 
	var maxiter = ui.commandUI.maxiter.integerValue;
	
	var sqrtSShh = Math.sqrt(S*S - h*h);
	var aParam = evalNewtRap ( aGuess, d, sqrtSShh, tol, maxiter )

//	In javascript, Math.log(n) returns ln(n).

//	This is Opalg/elbarto:
	var x1 = (0.5) * (aParam * Math.log((S + h)/(S - h)) - d);
	var x2 = (0.5) * (aParam * Math.log((S + h)/(S - h)) + d);
	var k = x1 - P1.x;
	var m = P1.y - aParam * cosh(x1/aParam);
//	yi = aParam * cosh((xi + k)/aParam) + m;

//	Plot the catenary, from xi = P1.x to P2.x.

	for ( var i = 0; i < numpoints; ++i )
	{
		var t = i / (numpoints - 1);

	var xi = P1.x + t * d;

	// Catenary formula.
	var yi = aParam * (Math.exp((xi + k) / aParam) + Math.exp((-1.0) *( xi + k) / aParam)) / 2.0 + m ; 
//	var yi = aParam * cosh((xi + k)/aParam) + m;
	
	var z = P1.z;
	var catPt = vm.createPoint( xi, yi, z );
	catfactory.createInput( 'point' );
	catfactory.setInput( catfactory.numInputs - 1, catPt );
}
	
	catfactory.update();
	
	return catfactory;
}

function DoCatenary2PtXY()
{
	// Pick the two suspension points.
	// Much code borrowed from CustomDistance script.

	var ui = moi.ui;
	var pointpicker = ui.createPointPicker();
	pointpicker.disableSnapToBasePt = true;
	
	if ( !GetPoint( pointpicker ) )
		return;

	var BasePt = pointpicker.pt;
	
	moi.ui.commandUI.g_BasePt = BasePt;
	pointpicker.bindFunc( moi.ui.commandUI.OnGetDistancePoint );	
	
	ui.beginUIUpdate();
	ui.hideUI( 'FirstPrompt' );
	ui.showUI( 'SecondPrompt' );
	ui.showUI( 'DistanceContainer' );
	ui.endUIUpdate();

	var linefactory = moi.command.createFactory( 'line' );
	linefactory.setInput( 0, BasePt );

	pointpicker.bindBasePt( linefactory.getInput(0) );
	pointpicker.bindResultPt( linefactory.getInput(1) );	
	
	if ( !GetPoint( pointpicker ) )
		return;	
		
	var OtherPt = pointpicker.pt;	
	var dist = moi.vectorMath.distance( BasePt, OtherPt );	
	
	if ( dist > 0.0 )
	{
		var SetVal = ui.commandUI.SetVal;
	
		SetVal( 'space', dist );
		
		SetVal( 'space', dist );
		
		ui.beginUIUpdate();
		ui.hideUI( 'SecondPrompt' );
		ui.hideUI( 'CancelContainer' );
		ui.showUI( 'DoneContainer' );
		
		ui.endUIUpdate();		

	}
	else {
		return;
	}		

//	The two points must have different x values.

	if ( BasePt.x != OtherPt.x )
	{
	}
	else
	{
		alert ("x values must not be equal")
		return;
	}
	
//	The two points must have the same z values.
	if ( BasePt.z != OtherPt.z )
	{
		alert ("z values must be equal")
		return;
	}

	var catfactory = Update(BasePt, OtherPt);

	var dlg = moi.ui.commandDialog;

	while ( 1 )
	{
		if ( !dlg.waitForEvent() )
		{
			catfactory.cancel();
			return false;
		}
			
		if ( dlg.event == 'done' )
			break;
			
		if ( dlg.event == 'S' || dlg.event == 'aGuess' || dlg.event == 'numpoints' || dlg.event == 'tol' || dlg.event == 'maxiter' )
		{
			catfactory.cancel();
			catfactory = Update(BasePt, OtherPt);
		}
	}	

	moi.ui.clearPickedPoints();
	linefactory.cancel();
	catfactory.commit();
}
	
DoCatenary2PtXY();
