//Catenary2PtXYZ is a script which produces a catenary between two picked points, in 3D space.
//	Gravity is in the negative y direction.

//	The z values may be different, in which case, a catFrame is created where
//	the z values of the two points are zero, and x axis is rotated 
//	perpendicular to the y-axis.

//	This script uses the Opalg math, and most of the elbarto code.

//	The MoI Distance script is used to select and measure the distance
//	between the points.  The user inputs the chain length, which must be
//	greater than the Distance, or a catenary will not form.

//	In the event that a catenary does not form, even though the chain length
//	is greater than the Distance, OR the catenary is upside down, 
//	Newton-Raphson has failed, and the user can adjust the GuessRadius
//	parameter, to be smaller or greater than 1.0.

//	For a chain length that is long in relation to the x-axis
//	distance, try a fractional GuessRadius.     
//	See PDF "Catenary Suspension from Two Points."

//	An ARCH seems to be formed if a negative GuessRadius is used.

#include "GetPoint.js"

function alert( msg )
{
	moi.ui.commandUI.alert( msg );
}

//	Define cosh and sinh.
function cosh(x)
{
	return (Math.exp(x) + Math.exp(-x))/2.0
}

function sinh(x)
{
	return (Math.exp(x) - Math.exp(-x))/2.0
}

//	Find aParameter with the Newton-Raphson Method of solving a Nonlinear Equation.
function evalNewtRap( aGuess, d, sqrtSShh, tol, maxiter )
{
	var cur_a = aGuess
	for ( var j = 0; j < maxiter; ++j )
	{
		var f_a = 2.0 * cur_a * sinh(d/(2.0*cur_a)) - sqrtSShh;
 		var Df_a = 2.0 * sinh(d/(2.0 * cur_a)) - d * cosh(d/(2.0 * cur_a)) /cur_a ;

		cur_a -= (f_a/Df_a);

		if (Math.abs(f_a) < tol)
		{
			return cur_a;
		}
	}
//	From elbarto code, check if maxiter exceeded:
//	Doesn't seem to do anything.	
//	if (j > maxiter)
//	{
//		alert ("maxiter exceeded, try fractional aGuess (or big aGuess)")
//	}
//	return;
}

//	Update the catenary.
function Update(BasePt, OtherPt)
{
	var ui = moi.ui;
	var vm = moi.vectorMath;
//	Set catFlag to false indicates P1.z does not equal P2.z.
	var catFlag = false;	
	var catfactory = moi.command.createFactory( 'curve' );

//	The two points have already been picked as different points, so
//	the Distance between them is already greater than zero.

//	The two points cannot be above each other.  Either the two x values,
//	or the two z values, or both x and z values, must be different.

//	The Point with the smaller x value is to be P1.
	if ( BasePt.x < OtherPt.x )
	{
		var P1 = BasePt;
		var P2 = OtherPt;
	}
	else if ( BasePt.x > OtherPt.x )
	{
		var P1 = OtherPt;
		var P2 = BasePt;	
	}
//	If x values are the same, point with smaller z value is to be P1.	
	else if ( BasePt.z < OtherPt.z) 
	{
		var P1 = BasePt;
		var P2 = OtherPt;
	}
	else if ( BasePt.z > OtherPt.z )
	{
		var P1 = OtherPt;
		var P2 = BasePt;	
	}		

//	if P1.z != P2.z create catFrame with CreateCatFrame function, and flag condition...
	if ( P1.z != P2.z )
	{
		var catFlag = true;
//	World coordinate frame is defined as the regular Moi screen frame.
//	To accomodate two endpoints which have different z values, a coordiate
//	frame called "catFrame", is used.  The catFrame has the vector direction
//	of the new y-axis parallel to the old y-axis.  The new x-axis is of course
//	perpendicular to both the old y-axis, and the new y-axis.
//  The new x-axis is rotated about the new y-axis, so that the catFrame.z
//	coordinates of the two picked points are zero.  Then the catenary is
//	calculated in the new xy catFrame, the points on the curve converted back
//	into world coordinates and the curve plotted.  The picture PNG1 titled
//	"creating a new coordinate frame" illustrates the problem.

//	The points will be converted to worldframe, and used to create the curve
//	on the screen, using the moi curve factory.

//	The catFrame FrameOrigin is at P1
//	P1 = BasePt =  (P1.x, P1.y, P1.z)
//	P2 = OtherPt = (P2.x, P2.y, P2.z)
//	P3 = 		   (P2.x, P1.y, P2.z)   (Note P1.y is correct, see PNG1)
//	The newXdir is the vector from P1 to P3.  To be used below.
	var P3 = vm.createPoint(P2.x, P1.y, P2.z)
	var newXdir = vm.makeVector( P1, P3);
	var newYdir = vm.createPoint(0,1,0);
//	Create the new catFrame coordinate frame.
	var catFrame = vm.createFrame( P1, newXdir, newYdir ); 

//	To convert to catFrame point, from world frame point :
	var cF_x = catFrame.distancex(P1);
	var cF_y = catFrame.distancey(P1);
	var cF_z = catFrame.distancez(P1);
	var P1 = vm.createpoint(cF_x,cF_y,cF_z);	

	var cF_x = catFrame.distancex(P2);
	var cF_y = catFrame.distancey(P2);
	var cF_z = catFrame.distancez(P2);
	var P2 = vm.createpoint(cF_x,cF_y,cF_z);
//	This is the end of the catFrame creation.  P1 and P2 are given in
//	catFrame coordinates with z = 0, and can be processed just like
//	coordinates in the xy plane, except the xi,yi,zi points need to
//	be translated back to worldFrame coordinates before added to the
//	catFactory.	
	}
	
//	By if/else above, d must be positive.
//	h could be positive or negative,and needs to be so.
	var d = (P2.x - P1.x);
	var h = (P2.y - P1.y);
	
//	S is the arclength of the catenary, and must be greater than Distance between P1 & P2.
//	S is the length of the chain, and the value of S must be provided by the user.
	var S = ui.commandUI.S.numericValue;
	
//	"aParam" is the radius of curvature at the vertex of the catenary, and needs to be
//	solved for, using a numeric method.  The Newton-Raphson method is used, and requires
//	an initial guess for aParam, called "aGuess", initially set to 1.0 units.  If the
//	catenary fails to apear, and S > Distance, then adjusting aGuess may help.
//	d*d/S is a good guess.
	var aGuess = ui.commandUI.aGuess.numericValue;
	
//	"numpoints" is the number of catenary points calculated, in order to plot the catenary.
//	The catfactory draws the catenary curve using control points, so the curve will be
//	very close to the control points, but may not be exactly on the points.
	var numpoints = ui.commandUI.numpoints.integerValue;

//	tolerance and maxiterations are used in the Newton-Raphson method, and can be adjusted.	
	var tol = ui.commandUI.tol.numericValue;
	tol /= 10000.0;	  //Divide tol by 10,000 for input field display. 
	var maxiter = ui.commandUI.maxiter.integerValue;
	
	var sqrtSShh = Math.sqrt(S*S - h*h);
	var aParam = evalNewtRap ( aGuess, d, sqrtSShh, tol, maxiter )

//	In javascript, Math.log(n) returns ln(n).
//	This is Opalg/elbarto math & code:
	var x1 = (0.5) * (aParam * Math.log((S + h)/(S - h)) - d);
	var x2 = (0.5) * (aParam * Math.log((S + h)/(S - h)) + d);
	var k = x1 - P1.x;
	var m = P1.y - aParam * cosh(x1/aParam);
//	yi = aParam * cosh((xi + k)/aParam) + m;

//	Plot the catenary, from xi = P1.x to P2.x.

	for ( var i = 0; i < numpoints; ++i )
	{
		var t = i / (numpoints - 1);
	
	var xi = P1.x + t * d;
	
//	var yi = aParam * cosh((xi + k)/aParam) + m;
	var yi = aParam * (Math.exp((xi + k) / aParam) + Math.exp((-1.0) *( xi + k) / aParam)) / 2.0 + m ; 
	var z = P1.z;
	var catPt = vm.createPoint( xi, yi, z );

//	To convert to worldframe point, from catFrame point:
//	(for example:) var worldPoint = catFrame.evaluate(cF.x, cF.y, cF.z);
	if (catFlag)
	{
	catPt = catFrame.evaluate(catPt.x, catPt.y, catPt.z);
	}

	catfactory.createInput( 'point' );
	catfactory.setInput( catfactory.numInputs - 1, catPt );
	}

	catfactory.update();
	
	return catfactory;
}

function DoCatenary2PtXYZ()
{
	// Pick the two suspension points.
	// Some code borrowed from CustomDistance script.

	var ui = moi.ui;
	var pointpicker = ui.createPointPicker();
	pointpicker.disableSnapToBasePt = true;
	
	if ( !GetPoint( pointpicker ) )
		return;

	var BasePt = pointpicker.pt;
	
	moi.ui.commandUI.g_BasePt = BasePt;
	pointpicker.bindFunc( moi.ui.commandUI.OnGetDistancePoint );	
	
	ui.beginUIUpdate();
	ui.hideUI( 'FirstPrompt' );
	ui.showUI( 'SecondPrompt' );
	ui.showUI( 'DistanceContainer' );
	ui.endUIUpdate();

	var linefactory = moi.command.createFactory( 'line' );
	linefactory.setInput( 0, BasePt );

	pointpicker.bindBasePt( linefactory.getInput(0) );
	pointpicker.bindResultPt( linefactory.getInput(1) );	
	
	if ( !GetPoint( pointpicker ) )
		return;	
		
	var OtherPt = pointpicker.pt;	
	var dist = moi.vectorMath.distance( BasePt, OtherPt );	
	
	if ( dist > 0.0 )
	{
		var SetVal = ui.commandUI.SetVal;
	
		SetVal( 'space', dist );
		
		ui.beginUIUpdate();
		ui.hideUI( 'SecondPrompt' );
		ui.hideUI( 'CancelContainer' );
		ui.showUI( 'DoneContainer' );
		
		ui.endUIUpdate();		

	}
	else {
		return;
	}		

//	The two points must have different x values OR different z values.

	if ( BasePt.x != OtherPt.x )
	{
	}
	else if ( BasePt.z != OtherPt.z )
	{
	}
	else
	{
	alert ("x values must differ, OR z values must differ")
		return;
	}
	
	var catfactory = Update(BasePt, OtherPt);

	var dlg = moi.ui.commandDialog;

	while ( 1 )
	{
		if ( !dlg.waitForEvent() )
		{
			catfactory.cancel();
			return false;
		}
			
		if ( dlg.event == 'done' )
			break;
			
		if ( dlg.event == 'S' || dlg.event == 'aGuess' || dlg.event == 'numpoints' || dlg.event == 'tol' || dlg.event == 'maxiter' )
		{
			catfactory.cancel();
			catfactory = Update(BasePt, OtherPt);
		}
	}	

	moi.ui.clearPickedPoints();
	linefactory.cancel();
	catfactory.commit();
}
	
DoCatenary2PtXYZ();
