// config: norepeat

#include "WaitForDialogDone.js"

/*
  Coordinates
  10/29/2020, Larry Fahnoe, fahnoe@FahnoeTech.com

  Creates leaders with either 2D or 3D coordinates. The script allows
  adjusting the origin and axes by establishing a CPlane in which case
  coordinates are relative to the CPlane. Display precision is
  controlled by Options > Dimensions > Formatting: Decimal
  display. Because the script can create multiple leaders, while the
  command is running Cancel or Undo removes leaders one at a time
  rather than removing everything just created. Leaders for the 2D
  views can optionally be prefixed with axis labels.

  Based on MoI's Leader and other MoI commands.
*/

const debug = false;
const version = '0.9';

function debugLog( msg) {
    var elapsed = Date.now() - start;
    moi.log( elapsed + 'ms: ' + msg + '\n');
}

/*
  Return a string with a point's coordinates. Coordinates are relative
  to either the world or CPlane origin. Depending upon view, return
  either 2D or 3D coordinates. Conditionally add axes labels.
*/
function pointCoordinates( digits, point) {
    var x, y, z;
    if ( moi.ui.commandUI.AdjustOrigin.value) {
        var cf = moi.view.getCPlane();
        x = cf.distancex( point);
        y = cf.distancey( point);
        z = cf.distancez( point);
    } else {
        x = point.x;
        y = point.y;
        z = point.z;
    }

    /*
      Avoid -0.000 in the coordinates which is caused by tiny negative
      values like -3.552713678800501e-15
    */
    var tinyNum = 1;
    for ( var i = 0; i < digits; ++i )
        tinyNum = tinyNum / 10;
    var zero = 0;
    zero = zero.toFixed( digits);
    x = ( Math.abs( x) < tinyNum) ? zero : x.toFixed( digits);
    y = ( Math.abs( y) < tinyNum) ? zero : y.toFixed( digits);
    z = ( Math.abs( z) < tinyNum) ? zero : z.toFixed( digits);

    var label = '';
    var delimiter = ', ';
    var labelAxes = moi.ui.commandUI.LabelAxes.value;
    switch( moi.ui.commandUI.view.value ) {
    case 'Top':
    case 'Bottom':
        if ( labelAxes)
            label = 'XY: ';
        return label + x + delimiter + y;
    case 'Front':
    case 'Back':
        if ( labelAxes)
            label = 'XZ: ';
        return label + x + delimiter + z;
    case 'Right':
    case 'Left':
        if ( labelAxes)
            label = 'YZ: ';
        return label + y + delimiter + z;
    case '3D':
        return x + delimiter + y + delimiter + z;
    }
}

/*
  Create a leader containing a point's coordinates. Loop to pick
  additional points for leaders.
*/
function coordinateLeader( pointpicker, leaders) {
    var fn = 'coordinateLeader() ';
    debugLog( fn);
    var presets = moi.geometryDatabase.getAnnotationPresets();
    var preset = moi.ui.commandUI.AnnotationPreset.value;
    var digits = presets[ preset].formattingDecimalPrecision;
    
    var factory = moi.command.createFactory( 'leader' );
    factory.setInput( 0, pointpicker.ptFrame );
    factory.setInput( 1, pointCoordinates( digits, pointpicker.pt) ); 
    factory.setInput( 2, preset); 

    moi.ui.beginUIUpdate();
    moi.ui.hideUI( 'BasePtPrompt' );
    moi.ui.showUI( 'NextPtPrompt' );
    moi.ui.endUIUpdate();

    pointpicker.projectToCPlane = false;
    pointpicker.disableVerticalStraightSnap = true;

    var projecttocplane = moi.ui.commandUI.ProjectToCPlane.value;

    if ( projecttocplane )
	pointpicker.restrictToPlane( pointpicker.cplane, true );
    else
	pointpicker.restrictToPlaneThroughPt( pointpicker.pt, false );

    factory.createInput( 'point' );
    var pointindex = factory.numInputs - 1;
    
    pointpicker.clearBindings();
    pointpicker.bindResultPt( factory.getInput( pointindex) );

    if ( !projecttocplane )
	pointpicker.bindResultOrientation( factory.getInput(0) );

    while ( true) {
	if ( !pointpicker.waitForEvent() ) {
            if ( debug)
                debugLog( fn + 'canceled: ' + pointpicker.cancelReason);
        }

        if ( debug)
            debugLog( fn + 'event: ' + pointpicker.event);

        if ( pointpicker.event == 'undo' || pointpicker.event == 'cancel' ) {
	    factory.removeLastInput();
            var numInputs = factory.numInputs;
            while ( numInputs-- > 2)
                moi.ui.removeLastPickedPoint();
            factory.cancel();
            return false;
        }

	if ( pointpicker.event == 'finished' ) {
	    factory.createInput( 'point' );
	    var pointindex = factory.numInputs - 1;
	    
	    pointpicker.clearBindings();
	    pointpicker.bindResultPt( factory.getInput( pointindex) );
	}

	if ( pointpicker.event == 'done' ) {
	    factory.removeLastInput();
	    factory.update();
	    break;
	}
    }

    leaders.push( factory.getCreatedObjects() );
    var numInputs = factory.numInputs;
    factory.commit();
    while ( numInputs-- > 2)
        moi.ui.removeLastPickedPoint();
    return true;
}

/*
  Get user options and conditionally establish a new CPlane to allow
  adjusting origin and axes. Loop to pick base points for leaders.
*/
function doCoordinates() {
    var fn = 'doCoordinates() ';
    debugLog( fn);
    if ( !WaitForDialogDone() )
        return;

    var savedcf;
    if ( moi.ui.commandUI.AdjustOrigin.value) {
        savedcf = moi.view.getCPlane();
        moi.view.setCPlaneInteractive();
    }

    moi.ui.beginUIUpdate();
    moi.ui.hideUI( 'OptionsPrompt' );
    moi.ui.hideUI( 'CoordinatesOptions' );
    moi.ui.endUIUpdate();

    var projecttocplane = moi.ui.commandUI.ProjectToCPlane.value;

    var leaders = new Array();
    moi.command.setCommandSpecificUndo( true );

    var exit = false;
    var pointpicker = moi.ui.createPointPicker();
    pointpicker.allowNestedCancel();
    while ( true) {
        moi.ui.beginUIUpdate();
        moi.ui.hideUI( 'NextPtPrompt' );
        moi.ui.showUI( 'BasePtPrompt' );
        moi.ui.endUIUpdate();

	if ( projecttocplane )
	    pointpicker.projectToCPlane = true;

        if ( exit) {
            if ( moi.ui.commandUI.AdjustOrigin.value) {
                moi.view.setCPlane( savedcf);
                debugLog( fn + 'restored CPlane');
            }
            return;
        }

	if ( !pointpicker.waitForEvent() ) {
            if ( debug)
                debugLog( fn + 'canceled: ' + pointpicker.cancelReason);
            exit = true;
        }

        if ( debug)
            debugLog( fn + 'event: ' + pointpicker.event);

        if ( pointpicker.event == 'done' || pointpicker.event == 'cancel') {
            exit = true;
            continue;
        }

        if ( pointpicker.event == 'undo' && leaders.length > 0 ) {
            moi.geometryDatabase.removeObjects( leaders.pop() );
            continue;
        }

        coordinateLeader( pointpicker, leaders);
        pointpicker.reset();
        pointpicker.allowNestedCancel();
    }
}

if ( moi.majorVersionNumber < 4)
    moi.ui.alert( 'Requires MoI version 4 or later');
else {
    if ( debug)
        moi.log( 'MoI: ' + moi.version + ', ' +
                 moi.command.currentCommandName + ': ' + version + '\n');
    var start = Date.now();
    doCoordinates();
    if ( debug)
        moi.ui.alert( moi.getLog() );
}
