// EggOvoid script by Brian McMillin, November, 2021.
// The EggOvoid script builds 3 types of egg-like profile curves, the Oviform curve, the Pyriform curve, and the Avianform curve.
// References:
// The primary paper is pay per view, but the extensive math
// documentation is a free download:
// https://nyaspubs.onlinelibrary.wiley.com/doi/10.1111/nyas.14680#:~:text=2-
// ,Conclusion,egg%20that%20exists%20in%20nature.

// The John D. Cook reference is also excellent, and brings up a possible problem of the change in value or position of the initial w and or B.
// https://www.johndcook.com/blog/2021/08/28/empirical-egg-formula/
//
// There are four shape parameters, 'L'ength, 'B'readth, 'w' which is the offset of the B axis from the y axis, and 'D' which is the diameter at x = L/4 from the pointy end).  The default parameters approximate a hens large egg, in millimeters.  Revolve of the closed curve profile produces solid.

function alert( msg )
{
	moi.ui.commandUI.alert( msg );
}

//  Making 'numpoints' an odd number may be preferable, as middle tx = 0.
//  'numpoints' is forced to be an odd number.
//  i ranges from 0 to (numpoints-1).
//  t = i / (numpoints - 1).
//	t is a fraction ranging from 0 to 1.
//	tx ranges from (-L/2 to L/2).
//  tx = L/2 * ((2*t) - 1)
//  The B_axis intersects the x_axis at x = -w. (At least it is supposed to.)  (This is the point (-w, 0, 0).)

function EvalOvoidCurve(t, LValue, BValue, wValue, DValue, buildtype, bottom)
{
	var L = LValue;
	var B = BValue;
	var w = wValue;
	var D = DValue;
	var tx = (L/2) * ((2*t) -1);	
	var LL = L*L;
	var ww = w*w;
	var n1 = LL - 4*tx*tx;
	var d1 = LL + 8*w*tx + 4*ww;
	var z = 0.0;	
	
	if ( buildtype == 'oviform' )
	{
//	var yy = B*B*0.25 * n1 / d1;
//	var y = Math.sqrt(yy);
	var y = B*0.5*Math.sqrt( n1 / d1 );
	if (bottom)
		y *= -1.0;
	}
	
	else if ( buildtype == 'pyriform' )
	{
	var d2 = 2*(L - 2*w)*tx*tx + (LL + 8*L*w - 4*ww)*tx + 2*L*ww + LL*w + LL*L;
//	var yy = B*B*0.25 * (n1*L / d2);
//	var y = Math.sqrt(yy);
	var y = B*0.5*Math.sqrt( n1*L / d2 );
	if (bottom)
		y *= -1.0;	
	}
	
	else if ( buildtype == 'avianform' )
	{
//	var sqrtI = Math.sqrt( n1/d1 );
//	var sqrtII = Math.sqrt( 5.5*LL + 11*L*w + 4*ww );
//	var sqrt3 = Math.sqrt( 3 );
//	var sqrtIV = Math.sqrt( LL + 2*w*L + 4*ww );
//	var d2 = 2*(L - 2*w)*tx*tx + (LL + 8*L*w - 4*ww)*tx + 2*L*ww + LL*w + LL*L;
//	var sqrtV = Math.sqrt( n1*L / d2 );
//	var d3 = sqrt3*B*L*( sqrtII - 2*sqrtIV );
//	var y = B*0.5*sqrtI*(1-(sqrtII *(sqrt3*B*L - 2*D*sqrtIV)/d3)*(1-sqrtV));

// JohnDCook math terms:	
	var k = Math.sqrt(5.5*LL + 11*L*w + 4*ww );
	k *= Math.sqrt(3)*B*L - 2*D*Math.sqrt(LL + 2*w*L + 4*ww );
	k /= Math.sqrt(3)*B*L;
	k /= Math.sqrt(5.5*LL + 11*L*w + 4*ww ) - 2*Math.sqrt( LL + 2*w*L + 4*ww );
	var a = 2*(L - 2*w);
	var b = LL + 8*L*w - 4*ww;
	var c = 2*L*ww + LL*w + LL*L;
	var t2 = L*(LL + 8*w*tx + 4*ww);
	t2 /= a*tx*tx + b*tx + c;
	var y = 0.5*B*Math.sqrt(LL - 4*tx*tx)/Math.sqrt(LL + 8*w*tx + 4*ww);
	y *= (1-k*(1-Math.sqrt(t2)));
	
	if (bottom)
		y *= -1.0;	
	}

	return moi.vectorMath.createPoint( tx, y, z );
}

function Update()
{
	var curvefactory = moi.command.createFactory( 'interpcurve' );

	var UI = moi.ui.commandUI;
	var LValue = UI.LValue.numericValue;
	var BValue = UI.BValue.numericValue;
	var wValue = UI.wValue.numericValue;
	var DValue = UI.DValue.numericValue;
	var numpoints = UI.numpoints.integerValue;
	var buildtype = UI.Build.value;
	var bottom = false;  // Do top half of curve first.

	if ( numpoints % 2 == 0)
		numpoints ++; // make numpoints odd.
	
	for ( var i = 0; i < numpoints; ++i )
	{
		var t = i / (numpoints - 1);
		
		var pt = EvalOvoidCurve(t, LValue, BValue, wValue, DValue, buildtype, bottom);
		
		curvefactory.createInput( 'point' );
		curvefactory.setInput( curvefactory.numInputs - 1, pt );	
	}

	// Countdown for bottom half of curve.	
	for ( var i = (numpoints-1); i >= 0; --i )
	{
		var t = i / (numpoints - 1);

		// Make bottom TRUE, for other half of curve.	
		var pt = EvalOvoidCurve(t, LValue, BValue, wValue, DValue, buildtype, !bottom);
		
		curvefactory.createInput( 'point' );
		curvefactory.setInput( curvefactory.numInputs - 1, pt );	
	}	

	curvefactory.update();
	return curvefactory;
}

function DoEggOvoid()
{
	var curvefactory = Update() 
	
	var dlg = moi.ui.commandDialog;
	
	while ( 1 )
	{
		if ( !dlg.waitForEvent() )
		{
			curvefactory.cancel();
			return false;
		}
			
		if ( dlg.event == 'done' )
			break;
			
		if ( dlg.event == 'LValue' || dlg.event == 'BValue' || dlg.event == 'wValue' || dlg.event == 'DValue' || dlg.event == 'numpoints' || dlg.event == 'Lslider'  || dlg.event == 'Bslider' || dlg.event == 'wslider'  || dlg.event == 'Dslider' || dlg.event == 'Build' )
		{
			
			curvefactory.cancel();
			
			curvefactory = Update();			
			
			moi.ui.commandUI.updating = false; //for slider update.		
		}		
	}
	curvefactory.commit();
}
DoEggOvoid();







