//	EulerArcSplineDraft3 script
//	Brian McMillin February 9, 2013.
 
//	This script creates a spiral curve, in the approximate shape of
//	the Euler spiral, also known as Spiros, Clothoid, and Cornu spiral.
//  As the distance between points decreases, the match with the Euler spiral improves. 

//	This script is based upon the paper Euler arc splines for curve
//	completion, by Hailing Zhou, Jianmin Zheng, and Xunnian Yang, available
//	here: www.math.zju.edu.cn/yxn/papers/EulerArcSplines.pdf.

#include "GetPoint.js"

function AddVectorPoints( v0, v1 )
{
	var x = (v0.x + v1.x);
	var y = (v0.y + v1.y);
	var z = (v0.z + v1.z);
	return moi.vectorMath.createPoint( x, y, z );
}

function Update( basept )
{
	var P0 = basept; //Start point of curve.
	var ui = moi.ui;
	var vm = moi.vectorMath;
	var factory = moi.command.createFactory( 'interpcurve' );
	var t0 = ui.commandUI.theta_zero.numericValue;
	var k1 = ui.commandUI.curvature.numericValue;
	var a  = ui.commandUI.alpha.numericValue;
	var s  = ui.commandUI.arclength.numericValue;
	var n  = ui.commandUI.numarcs.integerValue;	
	// t0 is in radians, s is positive distance.	
	var t0 = t0 * Math.PI;
	var s = Math.abs(s);
	if (s == 0.0)
	{
		s = 0.001;
	}
	var originPt = vm.createPoint( 0.0, 0.0, 0.0 );  
	// vPtNew holds the latest point on the curve, starting with P0.
	var vPtNew = vm.makeVector( originPt, P0 );
	var z0 = 0.0 ;
	// Save the basept in the curve factory.
	factory.createInput( 'point' );
	factory.setInput( factory.numInputs - 1, P0 );

	//vector point Pi = vector P(i-1) + the vector from P(i-1) to P(i).
	for ( var i = 1; i <= n; ++i )
	{
		var vPtOld = vPtNew ;
		var thetai = t0 + (i * k1 * s) + ((i-1) * i * s * s * a )/2 ;
		var phii = ((s*s*(i-1)*(i-1)*a) + ((2*i)-1) * k1 * s )/2 ;
		var ki = a * s * (i-1) + k1 ;
		var chord = (2 * Math.sin(ki * s * 0.5 )) / ki ;
		var deltax = chord * Math.cos( phii + thetai ) ;
		var deltay = chord * Math.sin( phii + thetai ) ;
		chordPt = vm.createPoint(deltax, deltay, z0 );
		var vBetweenPts = vm.makeVector( originPt, chordPt);
		vPtNew = AddVectorPoints( vPtOld, vBetweenPts ) ;

		PtNew = vm.createPoint( vPtNew.x, vPtNew.y, vPtNew.z );

		factory.createInput( 'point' );
		factory.setInput( factory.numInputs - 1, PtNew );
	}

	factory.update();
	
	return factory;	
}

function DoEulerArcSpline()
{

	var ui = moi.ui;
	var pointpicker = ui.createPointPicker();
//	pointpicker.disableSnapToBasePt = true;
	
	if ( !GetPoint( pointpicker ) )
		return;

	var basept = pointpicker.pt;
	var factory = Update(basept);	
	var dlg = ui.commandDialog;
	
	// Wait for any changes to the UI or cancel or done.
	while ( 1 )
	{
		if ( !dlg.waitForEvent() )
		{
			factory.cancel();
			return false;
		}
			
		if ( dlg.event == 'done' )
			break;
			
		if ( dlg.event == 'theta_zero' || dlg.event == 'curvature' || dlg.event == 'alpha' || dlg.event == 'arclength' || dlg.event == 'numarcs' )
		{
			factory.cancel();
			factory = Update(basept);
		}
	
	}

	moi.ui.clearPickedPoints();
	factory.commit();
}

DoEulerArcSpline();