//	V1FrenetClothoid3D_RK2 script.

//	Reference: "3D Euler Spirals for 3D Curve Completion" by Gur Harary
//	and Ayellet Tal. Also "Piecewise 3D Euler Spirals" and "Visualizing 3D Euler Spirals" 	http://cgm.technion.ac.il/people/Gur/

//	Script version 1 just makes a curve.  V1 does NOT do the gradient-descent,
//	which is described in the reference as, given two points, and their
//	associated tangents (x0, T0) , (x1, T1), find the four curvature and
//	torsion parameters, and Euler spiral Length, in order to connect the
//	two points with a segment of an Euler spiral.
 
//	This script uses Runge-Kutta2, midpoint method.
//	It is a modified version of FrenetNatLogSpiralRK2, which has a
//	pdf "Frenet-Serret Javascript program for Moment of Inspiration."
//	DRAFT by Brian McMillin February 25, 2012
//	Negative and zero values are allowed.  Zero torsion reverts to 2D.

function alert( msg )
{
	moi.ui.commandUI.alert( msg );
}

function crossProduct( v0, v1 )
{
	var x = (v0.y * v1.z) - (v0.z * v1.y);
	var y = (v0.z * v1.x) - (v0.x * v1.z);
	var z = (v0.x * v1.y) - (v0.y * v1.x);
	return moi.vectorMath.createPoint( x, y, z );
}

function scaleVector( v0, factor )
{
	var x = v0.x * factor;
	var y = v0.y * factor;
	var z = v0.z * factor;
	return moi.vectorMath.createPoint( x, y, z );
}	

function addVectors( v0, v1 )
{
	var x = v0.x + v1.x;
	var y = v0.y + v1.y;
	var z = v0.z + v1.z;
	return moi.vectorMath.createPoint( x, y, z );
}
	
function dotproduct( v0, v1 )
{
	return (v0.x * v1.x) + (v0.y * v1.y) + (v0.z * v1.z);
}
	
function lengthsq( v )
{
	return dotproduct ( v, v );
}

function normalize( v )
{
	var lensq = lengthsq(v);
	if ( lensq < 1.0e-12 )
		return; // degenerate
	var invlen = 1.0 / Math.sqrt(lensq);
//	v.x *= invlen;
//	v.y *= invlen;
//	v.z *= invlen;
	var x = v.x * invlen;
	var y = v.y * invlen;
	var z = v.z * invlen;
	return moi.vectorMath.createPoint( x, y, z );	
}

function Update(r_0, d_r, S_0, D_S, L, numPts)
{
	var clothoidfactory = moi.command.createFactory( 'curve' );	
	var vm = moi.vectorMath;   //abbreviation
	var gd = moi.geometryDatabase;  //abbreviation	

//	r_0 = initial radius of curvature. 
//	d_r = delta_r = change in radius of curvature along arc.
//	S_0 = sigma0 = initial radius of torsion.
//	D_S = delta_sigma = change in radius of torsion along arc.
//	L = total arc length.
//	si = arc length at index i.
//	K = K(si) = kappa(si) = curvature = (radius0 + (delta_r * si))
//	tau = tau(si) = torsion = (sigma0 + (delta_sigma * si))
//	ds is the interval over which to calculate the next value for each of
//  Tangent, Normal, and Binormal (P,T,N, and B).
//	ds = L/numPts.

// Initiallize.
	var Pti = vm.createPoint( 0.0, 0.0, 0.0 );
	var Ti = vm.createPoint( 1.0, 0.0, 0.0 );
	var Ni = vm.createPoint( 0.0, 1.0, 0.0 );
	var Bi = vm.createPoint( 0.0, 0.0, 1.0 );

//	Put the initial point in the clothoidfactory.	
	clothoidfactory.createInput( 'point' );
	clothoidfactory.setInput( clothoidfactory.numInputs - 1, Pti );

	var si = 0.0;
	var ds = L/numPts;
	var simp = si + ds/2;

	for ( var i = 0; i < numPts; ++i )
	{
		si = i * ds;
		simp = si + ds/2;
		
//		3D Clothoid Spiral.		
		var kappa = ( r_0 + (d_r * si ));
		var kappa2 = ( r_0 + (d_r * simp ));
		var tau = ( S_0 + (D_S * si ));
		var tau2 = ( S_0 + (D_S * simp ));		

//	alert("to here1")

// Midpoint slope values.
//		Tmp = Ti + Kappa * Ni * (ds/2);
			var Nik = scaleVector( Ni, kappa * ds/2 );
		var Tmp = addVectors( Ti, Nik );
			var Bit = scaleVector( Bi, (tau * ds/2) );
			var Tik = scaleVector( Ti, ((-1) * kappa * ds/2) );
		var Nmp = addVectors( Ni, Bit );
		Nmp = addVectors( Nmp, Tik ); 

			var Nit = scaleVector( Ni, ( (-1) * tau * ds/2 ) );
		var Bmp = addVectors( Bi, Nit );
		Tmp = normalize(Tmp);
		Nmp = normalize(Nmp);
		Bmp = normalize(Bmp);
		
//	alert("to here5")
		
// Values at end of interval, (i+1);
//		newT = Ti + (Kappa2*ds) * Nmp;
			var Nmpk = scaleVector( Nmp, kappa2 * ds );
		var newT = addVectors( Ti, Nmpk);
			
			var Bmpt = scaleVector( Bmp, tau2 * ds );
			var Tmpk = scaleVector( Tmp, ((-1) * kappa2 * ds ));
		var newN = addVectors( Ni, Bmpt );
		newN = addVectors( newN, Tmpk ); 
		
			var Nmpt = scaleVector( Nmp, ( (-1) * tau2 * ds )); 
		var newB = addVectors( Bi, Nmpt );
		normalize(newT);
		normalize(newN);
		normalize(newB); 

			var Tis = scaleVector( Ti, ds/6 );
			var Tmps = scaleVector( Tmp, 4*ds/6 );
			var newTs = scaleVector( newT, ds/6 );
		var newPti = addVectors( Pti, Tis );
		newPti = addVectors(newPti, Tmps );
		newPti = addVectors(newPti, newTs );
	
//	alert("to here6")
	
		Ti = newT;
		Ni = newN;
		Bi = newB;
		Pti = newPti;

		clothoidfactory.createInput( 'point' );
		clothoidfactory.setInput( clothoidfactory.numInputs - 1, Pti );
	}
	clothoidfactory.update();	

	return clothoidfactory;
}

function Natural3DSpiral()
{
	var clothoidfactory = Update( moi.ui.commandUI.r_0.numericValue,
	moi.ui.commandUI.d_r.numericValue,
	moi.ui.commandUI.S_0.numericValue,
	moi.ui.commandUI.D_S.numericValue,
	moi.ui.commandUI.L.numericValue,
	moi.ui.commandUI.numPts.integerValue );
	
	var dlg = moi.ui.commandDialog;
	
	while ( 1 )
	{
		if ( !dlg.waitForEvent() )
		{
			clothoidfactory.cancel();
			return false;
		}
			
		if ( dlg.event == 'done' )
			break;
			
		if ( dlg.event == 'r_0' || dlg.event == 'd_r' || dlg.event == 'S_0' || dlg.event == 'D_S' || dlg.event == 'L' || dlg.event == 'numPts' )
		{
		clothoidfactory.cancel();
		clothoidfactory = Update( moi.ui.commandUI.r_0.numericValue,
		moi.ui.commandUI.d_r.numericValue, moi.ui.commandUI.S_0.numericValue,
		moi.ui.commandUI.D_S.numericValue, moi.ui.commandUI.L.numericValue,
		moi.ui.commandUI.numPts.integerValue );
		}
	}
	
	clothoidfactory.commit();
}

Natural3DSpiral();