//	_FrenetKTCurve by Brian McMillin August, 2014.
//	This script creates a 3D space curve and ribbon, using the Frenet Frame and two 2D curves of kappa and tau.
//	The arclength distances ("s", or si) of the 3D curve to be created,
//	is plotted on the x-axis for the two 2D curves.
//	The Curvature (kappa) is plotted on the y-axis, to form the first 2D curve. 
//	The Torsion (tau) is also plotted on the y-axis, to form the second 2D curve.
//	The graphs of these two functions are NURBS curves, that satisfy the vertical line test.
//	(One value of s yields one value of kappa, and one value of tau.)
//	kappa equal to 0.0 will be an inflection point...(OR A PROBLEM?)
 
//  A Moebius strip centerline curve, based upon the physics of real materials,
//	has been calculated by E.L. Starostin, and G.H.M. van der Heijden, defined by Curvature (kappa)
//	and Torsion (tau). 

//	http://www.ucl.ac.uk/~ucesgvd/moebius.pdf (copy to browser)
//	http://www.homepages.ucl.ac.uk/~ucesgvd/moebius_suppl.pdf
//	http://arxiv.org/abs/0905.1880
//	http://arxiv.org/abs/0804.0419
//	http://arxiv.org/abs/0807.0377
//	http://onlinelibrary.wiley.com/doi/10.1002/pamm.200700858/pdf

//	This script does NOT solve the DAE equations to determine the Kappa and Tau curves for a
//	particular width of a Moebius strip of length 2*PI. (nor for a 3-fold Moebius strip.)
//	DAE solving may be attempted later.  Maple or Mathematica can do it with Runge-Kutta-Fehlberg45.

//	Using Moi View Image, the graphs of kappa and tau, from the Starostin/Heijden papers, were used to
//	approximately plot the kappa and tau curves for a Moebius strip with length 2PI, and a (half) width
//	of 0.8 units, to two NURBS curves of kappa and tau.
//	Integration of the 3 Frenet-Serret equations, needs the changing values of kappa and tau at various
//	distances s, to yield an interpcurve of the Moebius strip centerline, and plot the 3d ribbon
//	along the binormal or normal. Project, using the current value of si, as a Moi point (si, 0, 0),
//	yields the point on the 2D kappa curve, or the 2D tau curve,
//	whose y coordinate is the current value of kappa, or tau, respectively.   
//	This script is using Runge-Kutta4 method of integration, to solve the 3
//	differential equations for Tangent, Normal, and BiNormal, as described
//	in 3D-XplorMath-J.  http://3d-xplormath.org/j/source/BSDLicense.txt
//	The next point is obtained with Simpson integration. "Because of the two
//	Runge Kutta steps, dt is half the step size" (dt = h = ds)

// config: norepeat

#include "GetObjects.js"

function addVectors( v0, v1 )
{
	var x = v0.x + v1.x;
	var y = v0.y + v1.y;
	var z = v0.z + v1.z;
	return moi.vectorMath.createPoint( x, y, z );
}

function alert( msg )
{
	moi.ui.commandUI.alert( msg );
}

var g_style_index = 1;

function AssignColors( factory )
{
	var styles = moi.geometryDatabase.getObjectStyles();
	var curves = factory.getCreatedObjects();
//	for ( var i = 0; i < curves.length; ++i, ++g_style_index )
	for ( var i = 0; i < curves.length; ++i )	
	{
		if ( g_style_index == styles.length )
		{
			g_style_index = 0;
		}
		var curve = curves.item(i);
		curve.styleIndex = g_style_index;
	}
}

function Cancel( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].cancel();
	factories.length = 0;
}

function Commit( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].commit();
}

function crossProduct( v0, v1 )
{
	var x = (v0.y * v1.z) - (v0.z * v1.y);
	var y = (v0.z * v1.x) - (v0.x * v1.z);
	var z = (v0.x * v1.y) - (v0.y * v1.x);
	return moi.vectorMath.createPoint( x, y, z );
}

function dotproduct( v0, v1 )
{
	return (v0.x * v1.x) + (v0.y * v1.y) + (v0.z * v1.z);
}

function GetValue( name )
{
	var val = 0.0;
	
	var input = moi.ui.commandUI[name];
	if ( input.value != '' )
	{
		try	{
			val = input.numericValue;
		} catch(e){}
	}
	
	return val;
}

function lengthsq( v )
{
	return dotproduct ( v, v );
}

function normalize( v )
{
	var lensq = lengthsq(v);
	if ( lensq < 1.0e-12 )
		return; // degenerate
	var invlen = 1.0 / Math.sqrt(lensq);
	var x = v.x * invlen;
	var y = v.y * invlen;
	var z = v.z * invlen;
	return moi.vectorMath.createPoint( x, y, z );	
}

function scaleVector( v0, factor )
{
	var x = v0.x * factor;
	var y = v0.y * factor;
	var z = v0.z * factor;
	return moi.vectorMath.createPoint( x, y, z );
}

/*
//	Get kappa or tau, the y value of the projection point.
function EvalKappaTau( curvelist, si )
{
	var basept = moi.vectorMath.createPoint( si, 0, 0 );// Input 3
	var pointfactory = moi.command.createFactory( 'point' );
	pointfactory.setInput( 0, basept );
	var siPt = pointfactory.calculate();
	var siPtobjlist = moi.geometryDatabase.createObjectList();//Wrap	
	moi.geometryDatabase.addObjects( siPt );
	siPtobjlist.addObject( siPt.item(0) ); // Input 0 ObjectList
	moi.geometryDatabase.removeObjects( siPt );// GOOD
	var endDir = moi.vectorMath.createPoint( si, 1.0, 0.0 );// Input 4
	var projectfactory = moi.command.createFactory( 'project' );
	projectfactory.setInput( 0, siPtobjlist );
	projectfactory.setInput( 1, curvelist );
	projectfactory.setDefaultDirection();
	
	var projectmode = 'direction';
//	var done = false;
	projectfactory.setInput( 2, projectmode );
	projectfactory.setInput( 3, basept );
	projectfactory.setInput( 4, endDir );
//	projectfactory.setInput( 5, ______);// Delete Inputs
//	projectfactory.setInput( 6, ______);// cplane
	projectfactory.update();
	var ktpointobj = projectfactory.getCreatedObjects();
	projectfactory.commit();// GOOD
//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>	
	var ktpoint = ktpointobj.item(0);//If Cancel during Project, exception occurs here
//////////////////////////////////////////////////////////////////////////////	
	var kORtValue = ktpoint.pt.y; //(per Michael)
//	alert ( "kORtValue = " + kORtValue );
	moi.geometryDatabase.removeObjects( ktpointobj );// GOOD	
	return kORtValue;
}
*/

function CreateBiNormals( B, halfWidth, Pti, BothsidesB, bfactories )
		{
			linefactory = moi.command.createFactory( 'line' );		
			// Note, pt2 is the vector sum of Pti and B.
			var BRib = normalize(B); // Leave B unchanged.
			BRib = scaleVector( B, halfWidth);	
			var pt0 =  Pti   ;		
			var pt2 =  addVectors( Pti, BRib );		
			linefactory.setInput( 0, pt0 );
			linefactory.setInput( 1, pt2 );
			linefactory.setInput( 2, BothsidesB );		
			bfactories.push( linefactory );
			// The next codeline displays lines as they are created.
			linefactory.update();
		}

function CreateNormals( N, halfWidth, Pti, BothsidesN, nfactories )
		{
			line2factory = moi.command.createFactory( 'line' );		
			// Note, pt3 is the vector sum of Pti and N.
			var NRib = normalize(N); // Leave N unchanged.
			NRib = scaleVector( N, halfWidth);	
			var pt0 =  Pti   ;		
			var pt3 =  addVectors( Pti, NRib );		
			line2factory.setInput( 0, pt0 );
			line2factory.setInput( 1, pt3 );
			line2factory.setInput( 2, BothsidesN );		
			nfactories.push( line2factory );		
			line2factory.update();
		}

		
function Update( kappaobjlist, tauobjlist, cLength, numpoints, halfWidth, ShowBiNormals, BothsidesB, ShowNormals, BothsidesN, cfactories, bfactories, nfactories )
{
	Cancel( cfactories );
	Cancel( bfactories );
	Cancel( nfactories );	

//alert ("To Here1");
	
	var vm = moi.vectorMath;   //abbreviation
	var gd = moi.geometryDatabase;  //abbreviation
	numpoints = Math.abs(numpoints);
	
	// Frenet Frame.
	var Pti = vm.createPoint( 0.0, 0.0, 0.0 );
	var T = vm.createPoint( 1.0, 0.0, 0.0 );
	var N = vm.createPoint( 0.0, 1.0, 0.0 );
	var B = vm.createPoint( 0.0, 0.0, 1.0 );

	if ( ShowBiNormals )	
		CreateBiNormals( B, halfWidth, Pti, BothsidesB, bfactories );

	if ( ShowNormals )	
		CreateNormals( N, halfWidth, Pti, BothsidesN, nfactories );
	
	// RungeKutta4 stages.
	var dT1 = vm.createPoint( 0.0, 0.0, 0.0 );
	var dT2 = vm.createPoint( 0.0, 0.0, 0.0 );	
	var dT3 = vm.createPoint( 0.0, 0.0, 0.0 );	
	var dT4 = vm.createPoint( 0.0, 0.0, 0.0 );	
	var dN1 = vm.createPoint( 0.0, 0.0, 0.0 );
	var dN2 = vm.createPoint( 0.0, 0.0, 0.0 );	
	var dN3 = vm.createPoint( 0.0, 0.0, 0.0 );	
	var dN4 = vm.createPoint( 0.0, 0.0, 0.0 );	
	var dB1 = vm.createPoint( 0.0, 0.0, 0.0 );
	var dB2 = vm.createPoint( 0.0, 0.0, 0.0 );	
	var dB3 = vm.createPoint( 0.0, 0.0, 0.0 );	
	var dB4 = vm.createPoint( 0.0, 0.0, 0.0 );
	// For intermediate results.
	var vS1 = vm.createPoint( 0.0, 0.0, 0.0 );
	var vS2 = vm.createPoint( 0.0, 0.0, 0.0 );
	var vS3 = vm.createPoint( 0.0, 0.0, 0.0 );	
	var vS4 = vm.createPoint( 0.0, 0.0, 0.0 );	
	
	var si = 0.0;
	cLength *= Math.PI;
	var ds = 0.5*cLength/numpoints;
	var kappa;
	var tau;
	
//alert ("cLength = " + cLength + "\nsi = " + si + "\nds = " + ds );

//	Put the initial point in the cfactory.
	cfactory = moi.command.createFactory( 'interpcurve' );	
	cfactory.createInput( 'point' );
	cfactory.setInput( cfactory.numInputs - 1, Pti );
	cfactories.push( cfactory );	

//alert ("Pti.x = " + Pti.x + "\nPti.y = " + Pti.y + "\nPti.z = " + Pti.x );  	
//alert ("T.x = " + T.x + "\nT.y = " + T.y + "\nT.z = " + T.z );
//alert ("N.x = " + N.x + "\nN.y = " + N.y + "\nN.z = " + N.z );
//alert ("B.x = " + B.x + "\nB.y = " + B.y + "\nB.z = " + B.z );
	
	for ( var i = 0; i < numpoints; ++i )
	{

		for ( var j = 1; j < 3; ++j)
		{

		//	Get the values of kappa and tau from their 2D curves.
//			kappa =	EvalKappaTau( kappaobjlist, si );
//			tau = EvalKappaTau( tauobjlist, si );
			kappa =	moi.ui.commandUI.EvalKappaTau( kappaobjlist, si );
			tau = moi.ui.commandUI.EvalKappaTau( tauobjlist, si );

		//	This section is for the k1 step of RK4.	
		//	dT1 = N times (kappa*(ds/2));
			dT1 = scaleVector( N, kappa*ds/2 );

		//	dN1 = (T times (-kappa*(ds/2))) plus (B times (tau*(ds/2);
			vS1 = scaleVector( T, (-kappa*ds/2) );
			vS2  = scaleVector( B, (tau*ds/2) );
			dN1 = addVectors( vS1, vS2 );
			
		//  dB1 = (N times (-tau*(ds/2))
			dB1 = scaleVector( N, (-tau*ds/2) );
		
		//	This section is for the k2 and k3 steps of RK4.
			si = si + ds/2;
			
// alert ("cLength = " + cLength + "\nsi = " + si + "\nds = " + ds );
// alert ("kappa = " + kappa + "\ntau = " + tau );			
//alert ("Pti.x = " + Pti.x + "\nPti.y = " + Pti.y + "\nPti.z = " + Pti.x );  	
//alert ("T.x = " + T.x + "\nT.y = " + T.y + "\nT.z = " + T.z );
//alert ("N.x = " + N.x + "\nN.y = " + N.y + "\nN.z = " + N.z );
//alert ("B.x = " + B.x + "\nB.y = " + B.y + "\nB.z = " + B.z );
			
		//	Get the new values of kappa and tau from their 2D curves.
//			kappa =	EvalKappaTau( kappaobjlist, si );
//			tau = EvalKappaTau( tauobjlist, si );			
			kappa =	moi.ui.commandUI.EvalKappaTau( kappaobjlist, si );
			tau = moi.ui.commandUI.EvalKappaTau( tauobjlist, si );
			
			//	dT2 = (N plus dN1) times (kappa*ds);
			vS1 = addVectors( N, dN1 );
			dT2 = scaleVector( vS1, kappa*ds );
			
		//	dN2 = ((T plus dT1) times (-kappa*ds)) plus ((B plus dB1) times (tau*ds));
			vS1 = addVectors( T, dT1 );
			vS2 = scaleVector( vS1, -kappa*ds );

			vS1 = addVectors( B, dB1 );
			vS3 = scaleVector( vS1, tau*ds );
			dN2 = addVectors( vS2, vS3 );
			
		//	dB2 = ((N plus dN1) times (-tau*ds);
			vS1 = addVectors( N, dN1 );
			dB2 = scaleVector( vS1, -tau*ds);
			
		//	dT3 = (N plus(dN2 times (0.5))) times (kappa*ds);
			vS1 = scaleVector( dN2, 0.5 );
			vS2 = addVectors( N, vS1 );
			dT3 = scaleVector( vS2, kappa*ds );
			
		//	dN3 = ((T plus (dT2 times (0.5))) times ( -kappa*ds)) plus ((B plus (dB2 times (0.5))) times (tau*ds));
			vS1 = scaleVector( dT2, 0.5 );
			vS2 = addVectors( T, vS1 );
			vS3 = scaleVector( vS2, -kappa*ds );

			vS1 = scaleVector( dB2, 0.5 );
			vS2 = addVectors( B, vS1 );
			vS4 = scaleVector( vS2, tau*ds );
			dN3 = addVectors( vS3, vS4 ); 

		//	dB3 = (N plus (dN2 times (0.5))) times (-tau(t)*ds);
			vS1 = scaleVector( dN2, 0.5);
			vS2 = addVectors( N, vS1 );
			dB3 = scaleVector( vS2, -tau*ds);

		//	This section is for the k4 step of RK4.
			si = si + ds/2;
		//	Get the new values of kappa and tau from their 2D curves.
//			kappa =	EvalKappaTau( kappaobjlist, si );
//			tau = EvalKappaTau( tauobjlist, si );			
			kappa =	moi.ui.commandUI.EvalKappaTau( kappaobjlist, si );
			tau = moi.ui.commandUI.EvalKappaTau( tauobjlist, si );

		//	dT4 = (N plus dN3) times (kappa*ds/2);
			vS1 = addVectors( N, dN3 );
			dT4 = scaleVector( vS1, kappa*ds/2);
			
		//	dN4 = ((T plus dT3) times (-kappa*ds/2)) plus ((B plus dB3 ) times (tau*ds/2));
			vS1 = addVectors( T, dT3 );
			vS2 = scaleVector( vS1, -kappa*ds/2);
			
			vS1 = addVectors( B, dB3 );
			vS3 = scaleVector( vS1, tau*ds/2 );
			dN4 = addVectors( vS2, vS3 );
			
		//	dB4 = (N plus dN3) times (-tau*ds/2);		
			vS1 = addVectors( N, dN3 );
			dB4 = scaleVector( vS1, -tau*ds/2 );
			
		//	Combine the 4 stages of Runge Kutta 4:
		//	Pti = Pti plus (T times (ds*j*j/3.0));			
			vS1 = scaleVector( T, ds*j*j/3.0 );
			Pti = addVectors( Pti, vS1 );
						
		    vS1 = addVectors( dT1, dT2 );
			vS2 = addVectors( vS1, dT3 );
			vS3 = addVectors( vS2, dT4 );
			vS1 = scaleVector( vS3, 1.0/3.0 );
			T = addVectors( T, vS1 );
			
		    vS1 = addVectors( dN1, dN2 );
			vS2 = addVectors( vS1, dN3 );
			vS3 = addVectors( vS2, dN4 );
			vS1 = scaleVector( vS3, 1.0/3.0 );
			N = addVectors( N, vS1 );

		    vS1 = addVectors( dB1, dB2 );
			vS2 = addVectors( vS1, dB3 );
			vS3 = addVectors( vS2, dB4 );
			vS1 = scaleVector( vS3, 1.0/3.0 );
			B = addVectors( B, vS1 );
		}

	//	Pti = Pti plus (T times (ds/3.0))   Simpson integration.; 

		vS1 = scaleVector( T, ds/3.0 );
		Pti = addVectors( Pti, vS1 );

		cfactory.createInput( 'point' );
		cfactory.setInput( cfactory.numInputs - 1, Pti );
		cfactories.push( cfactory );
		cfactory.update();		

		if ( ShowBiNormals )
		{
			CreateBiNormals( B, halfWidth, Pti, BothsidesB, bfactories );
		}
		
		if ( ShowNormals )
		{
			CreateNormals( N, halfWidth, Pti, BothsidesN, nfactories )
		}	
	}			

	for ( var i = 0; i < cfactories.length; ++i )
		{
			cfactories[i].update();
		}
		
		for ( var i = 0; i < bfactories.length; ++i )
		{
			bfactories[i].update();
//			AssignColors( bfactories[i] );			
		}
		
		for ( var i = 0; i < nfactories.length; ++i )
		{
			nfactories[i].update();
			AssignColors( nfactories[i] );			
		}		
}

function DoFrenetKTCurve()
{
	var ui = moi.ui;
	var UI = moi.ui.commandUI;	
	var gd = moi.geometryDatabase;
	var dlg = moi.ui.commandDialog;	
	gd.deselectAll();
	
	var objectpicker = moi.ui.createObjectPicker();
	objectpicker.allowCurves();
	objectpicker.min = objectpicker.max = 2;	
//	objectpicker.allowStandaloneCurves();
	if ( !GetObjects( objectpicker ) )
		return;
		
	var curves = objectpicker.objects;		
	curves.sortBySelectionOrder();

	var kappacurve = curves.item(0);
	var taucurve = curves.item(1);
	
	// Must wrap each as objectlist for Project Input 1. (per Michael)
	var kappaobjlist = moi.geometryDatabase.createObjectList();	
	kappaobjlist.addObject( kappacurve );
	var tauobjlist = moi.geometryDatabase.createObjectList();	
	tauobjlist.addObject( taucurve );	

//	moi.geometryDatabase.deselectAll();	

	moi.ui.beginUIUpdate();
	moi.ui.hideUI( 'SelectPrompt' );
	moi.ui.showUI( 'OptionsPrompt' );
	moi.ui.showUI( 'Options' );
	moi.ui.endUIUpdate();

	var NeedUpdate = true;	
	
	var cfactories = new Array(); // centerline curve
	var bfactories = new Array(); // binormal ribbon lines
	var nfactories = new Array(); // normal ribbon lines	
	var cLength = GetValue( 'cLength' );
	var numpoints = UI.numpoints.integerValue;
	var halfWidth = GetValue( 'halfWidth' );
	var ShowBiNormalsCheckbox = UI.ShowBiNormalsCheckbox.value;	
	var BothsidesBCheckbox = UI.BothsidesBCheckbox.value;		
	var ShowNormalsCheckbox =  UI.ShowNormalsCheckbox.value;	
	var BothsidesNCheckbox = UI.BothsidesNCheckbox.value;	
	
	// Wait for cancel or done.	
	while ( 1 )
	{
		if ( !dlg.waitForEvent() )
		{
			Cancel( cfactories );
			Cancel( bfactories );
			Cancel( nfactories );			
			return;
		}
			
		if ( dlg.event == 'done' )
			break;
			
		if (  dlg.event == 'cLength' )
		{
		cLength = GetValue( dlg.event );
			NeedUpdate = true;
		}
		else if ( dlg.event == 'numpoints' )
		{
		numpoints = UI.numpoints.integerValue;
			NeedUpdate = true;
		}
		else if ( dlg.event == 'halfWidth' )
		{
		halfWidth = GetValue( dlg.event );
			NeedUpdate = true;
		}
		else if ( dlg.event == 'ShowBiNormalsCheckbox' )
		{
		ShowBiNormalsCheckbox = UI.ShowBiNormalsCheckbox.value;
			NeedUpdate = true;
		}
		else if ( dlg.event == 'BothsidesBCheckbox' )
		{
		BothsidesBCheckbox = UI.BothsidesBCheckbox.value;
			NeedUpdate = true;
		}
		else if ( dlg.event == 'ShowNormalsCheckbox' )
		{
		ShowNormalsCheckbox =  UI.ShowNormalsCheckbox.value;
			NeedUpdate = true;
		}
		else if ( dlg.event == 'BothsidesNCheckbox' )
		{
		BothsidesNCheckbox = UI.BothsidesNCheckbox.value;
			NeedUpdate = true;
		}		
		else if ( dlg.event == 'Update' )
		{		
			Update( kappaobjlist, tauobjlist, cLength, numpoints, halfWidth, ShowBiNormalsCheckbox, BothsidesBCheckbox, ShowNormalsCheckbox, BothsidesNCheckbox, cfactories, bfactories, nfactories );
			NeedUpdate = false;
		}
	}

	if ( NeedUpdate )	
			Update( kappaobjlist, tauobjlist, cLength, numpoints, halfWidth, ShowBiNormalsCheckbox, BothsidesBCheckbox, ShowNormalsCheckbox, BothsidesNCheckbox, cfactories, bfactories, nfactories );	
	
	Commit( cfactories );
	Commit( bfactories );
	Commit( nfactories );
}

DoFrenetKTCurve();
