#include "GetPoint.js"

/*
_GloboidLadder_beta002.  Beta version by Brian McMillin, Aug./Sept. 2018.

This script was written to create the envelope surfaces of the tooth flanks of the worm of a globoid worm gear, by creating a quad net.  The net enables MoI's Network command to be used to make the surfaces.  The TA (or ZA?) type of worm can be machined with a non-tilted trapezoidal lathe blade, or by CNC of a model.  The same globoid parametric equations as in the _GloboidHelix script are used.  The user selects two adjoining corner points of one edge of the trapezoid.  The script can be run again for another edge of the tooth flank trapezoid.

A curved ladder analogy is used for the net.  Another analogy would be a roller coaster train track, where the cross ties, which correspond to the ladder rungs, connect the two rails.  The net consists of 2 (or possibly more) rail spirals, and numerous rungs which are line segments.  Each rung line represents a cutting edge of the trapezoid tool, at a different rotation angle.  The number of rung lines is expressed as rungSpacing.  So, for example, if the numPoints is 600, and rung spacing is 2, then with the end rung, there would be 301 rungs (+/-).  If rungSpacing = 0, there are 2 rungs.  If rungSpacing = 1, there are 600 rungs, not 601 rungs.  (The number of rungs seems to vary by 1 or 2.)  Due to the changing curvature, Sweep does not form the proper surface. Lineweb does not create the proper rungs either.  rungSpacing needs to be high enough for Network to work.

The GloboidHelix script creates a spiral curve on the surface of a torus, or a portion of a torus.  The spiral is different from the ones created by the Toroidal Helix script.  The double enveloping hourglass worm drive uses globoid curved thread(s), around the central spindle area of a torus.

axisDistance is the distance between worm axis and gear wheel axis.
gearTeeth is the number of gear teeth.
wormTurns is the number of revolutions of the single worm thread.
(If wormTurns = gearTeeth, the entire torus will have a globoid spiral around it, and
it will no longer be a worm gear, but may be decorative.)
railSpirals is set to 2 globoid spirals, an adjoining pair.
rungSpacing reduces the number of rungs, down from a maximum of numpoints.
numpoints is the number of points plotted, for each globoid spiral.
For more wormTurns, numpoints can be set higher.
clockwise is the handedness of a globoid spiral.
thirdRail, for the midpoint spiral, is not implemented.
*/

function alert( msg )
{
	moi.ui.commandUI.alert( msg );
}

function Cancel( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].cancel();
		
	factories.length = 0;
}

function Commit( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].commit();
}

function EvalGloboidHelix( t, axisDistance, gearTeeth, wormStarts, wormTurns, flankPt, clw )
{
	var phi1 = Math.PI * 2.0 * wormTurns * t;  //worm rotation angle
	phi1 /= 2; //half of flanks above, half below.
	var aD = axisDistance; // distance between gear wheel axis and worm axis. 
	gearRatio = gearTeeth / wormStarts; // Example gearRatio = 48:1
	var phi2 = phi1 / gearRatio; // gear wheel rotation angle.
	var c1 = Math.cos(phi1);
	var s1 = Math.sin(phi1);
	var c2 = Math.cos(phi2);
	var s2 = Math.sin(phi2);
	var x1 = flankPt.x;
	var y1 = flankPt.y;
	var z1 = flankPt.z;
	
	var x = x1 * c1 - aD * s1 + aD * c2 * s1 + y1 * c2 * s1 + clw * z1 * s2 * s1;
	var y = -x1 * s1 - aD * c1 + aD * c1 * c2 + y1 * c2 * c1 + clw * z1 * s2 * c1;
	var z = -clw * aD * s2 + -clw * y1 * s2 + z1 * c2;
	return moi.vectorMath.createPoint( x, y, z );
}

function Update(flankPt1, flankPt2, factories )
{
	Cancel( factories );

	var interpfactory1 = moi.command.createFactory( 'interpcurve' );
	var interpfactory2 = moi.command.createFactory( 'interpcurve' );
	
	var axisDistance = moi.ui.commandUI.axisDistance.numericValue;
	var gearTeeth = moi.ui.commandUI.gearTeeth.integerValue;
//	var wormStarts = moi.ui.commandUI.wormStarts.integerValue;
	var wormStarts = 1;
	var wormTurns = moi.ui.commandUI.wormTurns.numericValue;
	var rungSpacing = moi.ui.commandUI.rungSpacing.integerValue;
	var numpoints = moi.ui.commandUI.numpoints.integerValue;
	var thirdRail = moi.ui.commandUI.thirdRail.value; //Not used.	
	var clockwise = moi.ui.commandUI.clockwise.value;
	var clw = 1.0; // right hand
	if ( clockwise )
	clw = -1.0; // left hand 
	var count = 0; // For rung spacing.
	
	for ( var i = 0; i < numpoints; ++i )
	{
		//var t = i / (numpoints - 1); // t from (0 to 1)
		
		var t = (i - (numpoints-1)/2) / ((numpoints - 1)/2); // t from (-1 to 1)
 
		var pt1 = EvalGloboidHelix( t, axisDistance, gearTeeth, wormStarts, wormTurns, flankPt1, clw );		
		var pt2 = EvalGloboidHelix( t, axisDistance, gearTeeth, wormStarts, wormTurns, flankPt2, clw );
//	Could add third rail from midpoint. Not implemented yet.
		
		// Build rung lines.		
		if ( count == 0 || i == ( numpoints - 1 ))
		{
			var linefactory = moi.command.createFactory( 'line' );
			linefactory.setInput( 0, pt1 );
			linefactory.setInput( 1, pt2 );
			factories.push( linefactory );			
			
			count = ( rungSpacing );	
		}	
		count -= 1;

		interpfactory1.createInput( 'point' );
		interpfactory1.setInput( interpfactory1.numInputs - 1, pt1 );
		
		interpfactory2.createInput( 'point' );
		interpfactory2.setInput( interpfactory1.numInputs - 1, pt2 );		
	}

	//  Save the rails to factories array.
	factories.push( interpfactory1 );
//	interpfactory1.update();	
	factories.push( interpfactory2 );
//	interpfactory2.update();
	
	for ( var i = 0; i < factories.length; ++i )
		factories[i].update();

}

function DoGloboidNet()
{
	var pointpicker = moi.ui.createPointPicker();
	if ( !GetPoint( pointpicker ) ) return;

	var flankPt1 = pointpicker.pt;

	moi.ui.beginUIUpdate();
	moi.ui.hideUI( 'SelectPrompt' );
	moi.ui.showUI( 'SelectPrompt2' );
	moi.ui.endUIUpdate();

	if ( !GetPoint( pointpicker ) )
		return;	
		
	var flankPt2 = pointpicker.pt;

	moi.ui.beginUIUpdate();
	moi.ui.hideUI( 'SelectPrompt2' );
	moi.ui.showUI( 'OptionsPrompt' );
	moi.ui.showUI( 'Options' );
	moi.ui.endUIUpdate();	
	
	var factories = new Array();
	
	Update(flankPt1, flankPt2, factories );

	// Wait for cancel or done.
	var dlg = moi.ui.commandDialog;
	while ( 1 )
	{
		if ( !dlg.waitForEvent() )
		{
			Cancel( factories );
			return;
		}
			
		if ( dlg.event == 'done' )
			break;
			
		if ( dlg.event == 'axisDistance' || dlg.event == 'gearTeeth' || dlg.event == 'wormTurns' || dlg.event == 'rungSpacing' || dlg.event == 'numpoints' || dlg.event == 'thirdRail' || dlg.event == 'clockwise' )

			Update(flankPt1, flankPt2, factories );
	}
	
	Commit( factories );
}

DoGloboidNet();
