//	Inversion Script by Brian McMillin March 4, 2014.
//	The Poincaré disk is a model for Hyperbolic Geometry, described within Euclidean geometry, a circle of radius r.
//	This transformation script is a reflection with respect to the circle.
//	Reference: http://www.ma.utexas.edu/users/gilbert/M333L/chp5vers4.pdf 
//	The center of the Poincaré circle of inversion, would invert to infinity, division by zero,
//	so curves through the center are best avoided.
//	The circle itself represents the boundary of the hyperbolic space, at infinity
//	The radius of the circle is the radius of inversion.
//	The script maps a point inside the circle, not equal to the center, to a point outside of the circle.
//	Or vice versa.
//	Much code from LineWeb, RingCircle, and BisectorLine was used.

// config: norepeat

#include "GetPoint.js"
//#include "WaitForDialogDone.js"
#include "GetObjects.js"

function Cancel( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].cancel();
		
	factories.length = 0;
}

function Commit( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].commit();
}

function dotproduct( v0, v1 )
{
	return (v0.x * v1.x) + (v0.y * v1.y) + (v0.z * v1.z);
}

function InvertPt( centerPt, evalPt, pR )
{
	var vm = moi.vectorMath;
	var dirOP = vm.makeVector(centerPt, evalPt);  // to be vector direction.
	var lensqOP = lengthsq( dirOP );  // length squared.
	var pRsq = pR * pR;  //radius squared.
	var mult = pRsq / lensqOP;  //combine the math.
//	The following 7 lines explain the combination of the formulas.	(O is origin, P is point evalPt.)
//	To make dirOP a unit vector, its x,y,z components would be divided by lenOP, its magnitude or norm or length.  
//	Also, lenOPinv, the length of the inversion vector for each evalPT, is pR * pR / lenOP.
//	So it is not necessary to take the square root of lensqOP, as the square is used to calculate mult.
//	var lenOP = Math.sqrt( lensqOP );
//	var lenOPinv = pR*pR/lenOP;  //Inversion formula.
//	return vm.createPoint( centerPt.x + (dirOP.x / lenOP) * lenOPinv, centerPt.y + (dirOP.y / lenOP) * lenOPinv, centerPt.z + (dirOP.z / lenOP) * lenOPinv );	
//	return vm.createPoint( centerPt.x + dirOP.x * pRsq / lensqOP, centerPt.y + dirOP.y * pRsq / lensqOP, centerPt.z + dirOP.z * pRsq / lensqOP);
	return vm.createPoint( centerPt.x + dirOP.x * mult, centerPt.y + dirOP.y *mult, centerPt.z + dirOP.z * mult );	
}

function lengthsq( v )
{
	return dotproduct( v, v );
}

//	Not used:
function normalize( v )
{
	var lensq = lengthsq( v );
	if ( lensq < 1.0e-12 )
		return; // degenerate
		
	var invlen = 1.0 / Math.sqrt(lensq);
	
	v.x *= invlen;
	v.y *= invlen;
	v.z *= invlen;
}

function Update( curves, centerPt, pRadius, numpoints, factories )
{
	Cancel( factories );

//	Show the Poincaré disk circle in top view.
	var circleFactory = moi.command.createFactory( 'circle' );
	var FrameOrigin = moi.vectorMath.createPoint( 0, 0, 0 );
	var FrameXAxis = moi.vectorMath.createPoint( 1, 0, 0 );
	var FrameYAxis = moi.vectorMath.createPoint(0, 1, 0 );
	var Frame = moi.vectorMath.createFrame( FrameOrigin, FrameXAxis, FrameYAxis );
	
	// Set the frame on the factory.
	circleFactory.setInput( 1, Frame );
	// Set the flag to tell the factory to use radius.
	circleFactory.setInput( 0, true );
	// Set the diameter on the factory and update it to generate the initial output.
	circleFactory.setInput( 3, pRadius );
	circleFactory.update();
	
	// Build points

	var points_for_crv = new Array( curves.length );
	
	for ( var iCrv = 0; iCrv < curves.length; ++iCrv )
	{
		crv = curves.item(iCrv);

		// Create point object to array.
		var pointfactory = moi.command.createFactory( 'point' );
		pointfactory.setInput( 0, crv.getStartPt() );
		var startpt = pointfactory.calculate();
		
		// Array it along the curve to produce a list of point objects.
		var arrayfactory = moi.command.createFactory( 'arraycurve' );
		arrayfactory.setInput( 0, startpt );     // Object list to array
		arrayfactory.setInput( 1, crv );        // Path curve for array
		arrayfactory.setInput( 2, 'NumItems' ); // Array mode
		arrayfactory.setInput( 3, numpoints );
		var arrayed_pts = arrayfactory.calculate();
		
		var pts = new Array();
		pts.push( startpt.item(0).pt );
		for ( var iPts = 0; iPts < arrayed_pts.length; ++iPts )
			pts.push( arrayed_pts.item(iPts).pt );
		if ( crv.isClosed )
			pts.push( startpt.item(0).pt );  // Needed to close new curve.

		//	Invert each point not equal to centerPt, to form interpcurve.
		var factory = moi.command.createFactory( 'interpcurve' );		
		for ( var i = 0; i < pts.length; ++i )
		{
			var evalPt = pts[i];
//	The point must not be the same as the origin.
			if ( evalPt.x != 0 || evalPt.y != 0 || evalPt.z != 0 )
			{
				var pt = InvertPt( centerPt, evalPt, pRadius );
				factory.createInput( 'point' );
				factory.setInput( factory.numInputs - 1, pt );
			}
		}
		factories.push( factory );
	}
	for ( var i = 0; i < factories.length; ++i )
	{
		factories[i].update();
	}	
}

function DoInversion()
{
	var ui = moi.ui.commandUI;
	var vm = moi.vectorMath;

/*	//  Picking the origin has been disabled.
	var pointpicker = moi.ui.createPointPicker();
	if ( !GetPoint( pointpicker ) )
	return;
	var centerPt = pointpicker.pt;	
*/
//  Picking center point becomes tedious, so set centerPt to origin.
	var centerPt = vm.createPoint( 0, 0, 0 ); 

	var ObjectPicker = moi.ui.createObjectPicker();
	ObjectPicker.allowStandaloneCurves();
	if ( !GetObjects( ObjectPicker ) )
		return;

	var curves = ObjectPicker.objects;
	curves.sortBySelectionOrder();

	moi.ui.beginUIUpdate();
	moi.ui.hideUI( 'SelectPrompt' );
	moi.ui.showUI( 'OptionsPrompt' );
	moi.ui.showUI( 'Options' );
		
	moi.ui.endUIUpdate();	

	var factories = new Array();
	
	Update( curves, centerPt, ui.pRadius.numericValue, ui.numpoints.integerValue, factories );

	var dialog = moi.ui.commandDialog;

	while ( 1 )
	{
		if ( !dialog.waitForEvent() )
		{
			Cancel( factories );
			return;
		}
			
		if ( dialog.event == 'done' )
			break;
			
		if ( dialog.event == 'pRadius' || dialog.event == 'numpoints' )
			Update( curves, centerPt, ui.pRadius.numericValue, ui.numpoints.integerValue, factories );
	}
			Commit( factories );
}

DoInversion();
