//	InversionCenterFast Script based upon the Poincaré disk model of Hyperbolic Geometry.
//	Brian McMillin March 14, 2014.

//	This version permits the circle of inversion to be chosen.  There are two option boxes.
//	The center point is selected, with the checkbox option of selecting the radius point.
//	Alternatively, the radius of the inversion circle may be entered.

//	This version does not incorporate a 3-point circle selection option, which will in a handy script named Inversion3pts. 

//	The circle of inversion may be different from the Poincaré disk circle.
//	An example of this is Hyperbolic tessellation, such as in Escher Circle Limit art.

//	The script will either invert curves selected by the user, or, if the selection box for point inversion is checked,
//	then the user may instead select points to be inverted.
//	If the user wished to preserve the vertexes of polygons, they should be separated first, with the MoI Separate command.
//	Otherwise more irregular curves may be produced, if a point does not happen to be array-ed to some vertex.
	
//	The script is primarily intended for the Top View, but some 3d effects may be attained in some cases.

//	Reference: http://www.ma.utexas.edu/users/gilbert/M333L/chp5vers4.pdf 
//	If the Poincaré circle is being used for the inversion, the center would invert to infinity,
//	so curves passing through the center are best avoided.

//	Much code from LineWeb, RingCircle, and BisectorLine, _SpherePoints, etcetera were used.

#include "GetObjects.js"
#include "GetPoint.js"

function DoInversion()
{
	var dialog = moi.ui.commandDialog;
	var ui = moi.ui.commandUI;
	var vm = moi.vectorMath;

	var pointpicker = moi.ui.createPointPicker();
	if ( !GetPoint( pointpicker ) )
	return;
	var centerPt = pointpicker.pt;	

	if (ui.pickRadius.value)
	{
		moi.ui.beginUIUpdate();
		moi.ui.hideUI( 'PickPrompt' );
		moi.ui.showUI( 'RadiusPrompt' );
		moi.ui.endUIUpdate();	

		var pointpicker = moi.ui.createPointPicker();
		if ( !GetPoint( pointpicker ) )
		return;
		var radiusPt = pointpicker.pt;
		
		moi.ui.beginUIUpdate();		
		moi.ui.hideUI( 'RadiusPrompt' );
		moi.ui.endUIUpdate();		
	}
	
	var invPtsFlag = ui.invertPoints.value;	
	if (invPtsFlag)
	{
		moi.ui.beginUIUpdate();
		moi.ui.hideUI( 'PickPrompt' );
		moi.ui.showUI( 'PointsPrompt' );
		moi.ui.endUIUpdate();	

		var ObjectPicker = moi.ui.createObjectPicker();
		ObjectPicker.allowPointObjects();
		if ( !GetObjects( ObjectPicker ) )
		return;

		var objectsToInvert = ObjectPicker.objects;
		objectsToInvert.sortBySelectionOrder();
		//  objectsToInvert = points, in this case.
		
		moi.ui.beginUIUpdate();		
		moi.ui.hideUI( 'PointsPrompt' );
		moi.ui.endUIUpdate();
	}
	else
	{
		moi.ui.beginUIUpdate();
		moi.ui.hideUI( 'PickPrompt' );
		moi.ui.showUI( 'CurvesPrompt' );
		moi.ui.endUIUpdate();	

		var ObjectPicker = moi.ui.createObjectPicker();
		ObjectPicker.allowStandaloneCurves();
		if ( !GetObjects( ObjectPicker ) )
		return;
		
		var objectsToInvert = ObjectPicker.objects;
		objectsToInvert.sortBySelectionOrder();
		//  objectsToInvert = curves, in this case.		

		moi.ui.beginUIUpdate();
		moi.ui.hideUI( 'CurvesPrompt' );
		moi.ui.showUI( 'OptionsPrompt' );
		moi.ui.showUI( 'Options' );
		moi.ui.endUIUpdate();	
	}
	
	if (ui.pickRadius.value)
	{
		var radiusValue = moi.vectorMath.distance( centerPt, radiusPt );
	}	
	else
	{
		var radiusValue = ui.pRadius.numericValue;
	}

	var objectsList = moi.ui.commandUI.Update( objectsToInvert, invPtsFlag, centerPt, radiusValue, ui.numpoints.integerValue );
	moi.geometryDatabase.addObjects(objectsList);

	while ( true )
	{
		dialog.waitForEvent();			
		if ( dialog.event == 'done' ) break;
			
		if ( dialog.event == 'pRadius' || dialog.event == 'numpoints' )
		{
			var radiusValue = ui.pRadius.numericValue;
			moi.geometryDatabase.removeObjects(objectsList);			
			var objectsList = moi.ui.commandUI.Update( objectsToInvert, invPtsFlag, centerPt, radiusValue, ui.numpoints.integerValue );
			moi.geometryDatabase.addObjects(objectsList);
		}
		else
		{
			moi.geometryDatabase.removeObjects(objectsList);			
			return
		}
	}	
}

DoInversion();
