
// RotateArray is a command that will produce a progressive rotation on the result of an array
// command. A point object must be included in the array, the point is used as the rotation origin.

function Update( Factories, TotalRotationAngle )
{
	for ( var i = 0; i < Factories.length; ++i )
	{
		var t = i / ( Factories.length - 1 );

		// Set the angle on the factory
		Factories[i].setInput( 2, t * TotalRotationAngle );
		
		// Update the factory to make it recalculate the rotated result.
		Factories[i].update();
	}
}

function DoRotateArray()
{
	// Get all selected objects.
	var Objects = moi.geometryDatabase.getSelectedObjects();
	
	// Get all the point objects out of the selected objects.
	var Points = Objects.getPoints();
	
	var NumPoints = Points.length;
	if ( NumPoints == 0 )
		return;
	
	// Calculate the number of objects in one single array instance, expecting
	// this to be a multiple of the number of point objects.
	var ObjectSetSize = Math.floor( Objects.length / NumPoints );
	if ( NumPoints * ObjectSetSize != Objects.length )
		return;
		
	// Create an array of rotate factories, one per object set.
	var NumSets = Objects.length / ObjectSetSize;
	var Factories = new Array( NumSets );

	var Counter = 0;
	
	for ( var i = 0; i < NumSets; ++i )
	{
		var ObjectSet = moi.geometryDatabase.createObjectList();
		var OriginPt = null;
	
		for ( j = 0; j < ObjectSetSize; ++j )
		{
			var Obj = Objects.item(Counter);
			++Counter;
			
			if ( Obj.isPointObject )
			{
				// Point objects have a "pt" property that gets the x,y,z point. Get that
				// as the rotation origin.
				OriginPt = Obj.pt;
			}
			else
			{
				// Not a point object, add it to the object list to be rotated.
				ObjectSet.addObject( Obj );
			}
		}

		if ( ObjectSet.length == 0 || OriginPt == null )
			return;

		// We've now got the objects to be rotated and the origin point for this set,
		// create the factory and fill these in.
		var Factory = moi.command.createFactory( 'rotate' );
		Factory.setInput( 0, ObjectSet );
		var frame = moi.ui.getActiveViewport().cplane;
		frame.origin = OriginPt;
		Factory.setInput( 1, frame );
		Factories[i] = Factory;
	}

	// Get the values from the UI controls.
	var TotalRotationAngle = moi.ui.commandUI.angle.value;
	
	// Show the initial result.
	Update( Factories, TotalRotationAngle );
	
	// Wait for any changes to the UI or cancel or done.
	var dialog = moi.ui.commandDialog;
	
	while ( 1 )
	{
		if ( !dialog.waitForEvent() )
			return; // Canceled.
			
		if ( dialog.event == 'done' )
			break; // "Done" pushed.
			
		if ( dialog.event == 'angle' )
		{
			// angle control changed.
			Update( Factories, moi.ui.commandUI.angle.value );
		}
	}
	
	// All done, commit the factories.
	for ( var i = 0; i < Factories.length; ++i )
		Factories[i].commit();
		
	// Remove the point objects.
	moi.geometryDatabase.removeObjects( Points );
	
}

DoRotateArray();
