/*
_squircularCyl script, (non-uniform), by Brian McMillin 2/13/2022.
 
This script produces a square to round cylinder, as per page 27 of the paper by Fong for the Fernandez-Guasti squircle, which is an intermediate shape between the circle and the square:
https://arxiv.org/ftp/arxiv/papers/1604/1604.02174.pdf
(For a uniform squircular cylinder, just extrude one squircle.)
c = zMax, the length of the cylinder, (along z axis.)
tau = z/c;  // linear.
zVal = tau * zMax;
tau varies from [1 to 0]; //(adjust 0 to 0.000001)
*/

function alert( msg )
{
	moi.ui.commandUI.alert( msg );
}

function Cancel( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].cancel();
		
	factories.length = 0;
}

function Commit( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].commit();
}

function Polar2Cartesian( theta, rho, zVal )
{
	// var z = 0.0;
	var x = rho * Math.cos(theta);
	var y = rho * Math.sin(theta);
	return moi.vectorMath.createPoint( x, y, zVal );	
}

function eval_rho(theta, radval, root2, sqfactor, sinval, zVal)
{
	var rho = ((radval * root2) / (sqfactor * Math.abs(sinval))) * Math.sqrt(1 - Math.sqrt( 1 - sqfactor * sqfactor * sinval*sinval ));
	// var z = 0.0;
	var x = rho * Math.cos(theta);
	var y = rho * Math.sin(theta);
	return moi.vectorMath.createPoint( x, y, zVal );
	//var pt = Polar2Cartesian( theta, rho );		
	factory.createInput( 'point' );
	factory.setInput( factory.numInputs - 1, pt );	
}

function Update( factories )
{
	Cancel( factories );
	var UI = moi.ui.commandUI;
	var zMax = UI.zMax.numericValue; // 4
	var numpointsZ = UI.numpointsZ.integerValue;	
	// say numpointsZ = 11;
	// j = (0, 1, 2, ... 10)
  for ( var j = 0; j < numpointsZ; ++j )		
  {
	var tau = j / (numpointsZ - 1); //(0/10, 1/10, ...10/10)
	var zVal = tau * zMax;
	// Put the square on the bottom:
	var tau = (1 - tau); // (1, 9/10, 8/10, ..., 0)
	if (tau === 0.0)
		tau = 0.000001
	var factory = moi.command.createFactory( 'curve' );
	var halfPI = Math.PI/2;
	var fullPI = Math.PI;
	var root2 = Math.sqrt(2);
	var tworoot2 = 2*root2;
	var threefactor = (3-tworoot2);
	var twofactor = (2-tworoot2);
	var onefactor = (1-root2);
	var radval = UI.radval.numericValue;
	//var tau = UI.tau.numericValue;	
	var numpoints = UI.numpoints.integerValue;

  //For tau = 0.000001 to 1.0
	var sqfactor = 2 * Math.sqrt(threefactor * tau * tau - twofactor * tau) / Math.pow((1 - onefactor * tau),2);
	var rho;
	var theta = 0; // theta Є [0 , 2PI]

	// Case theta = 0;
	rho = radval;
	var pt = Polar2Cartesian( theta, rho, zVal );		
	factory.createInput( 'point' );
	factory.setInput( factory.numInputs - 1, pt );

	// Case theta Є (0 , PI/2) // excludes endpoints.
	// for ( var i = 0; i < numpoints; ++i )
	for ( var i = 1; i < numpoints; ++i )		
	{
		var t = i / (numpoints); // not (numpoints - 1).		
		theta = t * halfPI;
		sinval = Math.sin(2 * theta); // do not add epsilon.
		var pt = eval_rho( theta, radval, root2, sqfactor, sinval, zVal );
		factory.createInput( 'point' );
		factory.setInput( factory.numInputs - 1, pt );
	}	
		
	// Case theta = PI/2.
	rho = radval;
	theta = halfPI;
	var pt = Polar2Cartesian( theta, rho, zVal );		
	factory.createInput( 'point' );
	factory.setInput( factory.numInputs - 1, pt );

	// Case theta Є (PI/2 , PI) // excludes endpoints.
	for ( var i = 1; i < numpoints; ++i )		
	{
		var t = i / (numpoints); // not (numpoints - 1).		
		theta = (t * halfPI) + halfPI;
		sinval = Math.sin(2 * theta); // do not add epsilon.
		var pt = eval_rho(theta, radval, root2, sqfactor, sinval, zVal);
		factory.createInput( 'point' );
		factory.setInput( factory.numInputs - 1, pt );
	}
	
	// Case theta = PI.
	rho = radval;
	theta = fullPI;
	var pt = Polar2Cartesian( theta, rho, zVal );		
	factory.createInput( 'point' );
	factory.setInput( factory.numInputs - 1, pt );

	// Case theta Є (PI , 3*PI/2) // excludes endpoints.	
	for ( var i = 1; i < numpoints; ++i )		
	{
		var t = i / (numpoints); // not (numpoints - 1).		
		theta = (t * halfPI) + fullPI;
		sinval = Math.sin(2 * theta); // do not add epsilon.
		var pt = eval_rho(theta, radval, root2, sqfactor, sinval, zVal);
		factory.createInput( 'point' );
		factory.setInput( factory.numInputs - 1, pt );
	}

	// Case theta = 3PI/2.
	rho = radval;
	theta = Math.PI + halfPI;
	var pt = Polar2Cartesian( theta, rho, zVal );		
	factory.createInput( 'point' );
	factory.setInput( factory.numInputs - 1, pt );

	// Case theta Є (3PI/2 , 2PI) // excludes endpoints.
	for ( var i = 1; i < numpoints; ++i )		
	{
		var t = i / (numpoints); // not (numpoints - 1).		
		theta = (t * halfPI) + fullPI + halfPI;
		sinval = Math.sin(2 * theta); // do not add epsilon.
		var pt = eval_rho(theta, radval, root2, sqfactor, sinval, zVal);
		factory.createInput( 'point' );
		factory.setInput( factory.numInputs - 1, pt );
	}

	// Case theta = 2PI.
	rho = radval;
	theta = 2 * fullPI;
	var pt = Polar2Cartesian( theta, rho, zVal );		
	factory.createInput( 'point' );
	factory.setInput( factory.numInputs - 1, pt );	

	//factory.update();
	factories.push( factory );
  }

	for ( var i = 0; i < factories.length; ++i )
		factories[i].update();  
}

function DoSquircleFG()
{
	var factories = new Array();
	
	Update( factories );

	var dlg = moi.ui.commandDialog;
	
	while ( 1 )
	{
		if ( !dlg.waitForEvent() )
		{
			Cancel( factories );
			return;
		}
			
		if ( dlg.event == 'done' )
			break;
			
		if ( dlg.event == 'radval' || dlg.event == 'radslider' || 'zMax' || dlg.event == 'zMzxslider' || dlg.event == 'numpoints' || dlg.event == 'numpointsZ' )
		{	
			Update( factories );
			moi.ui.commandUI.updating = false;
		}	
	}
	
	Commit( factories );
}

DoSquircleFG();
