/*
_TruchetTile script, Brian McMillin
Nov 7-24, 2017.

Truchet Tiles, which have simple 2D surface designs, are placed at random into cells of a Grid, organized into rows and columns, to form interesting random patterns.  This script has provision for 5 sets of truchet tiles, each with 4 source tiles, namely Duotone plus, with wide quarter circle arcs, Wide Curves, with quarter circle arcs alone, Duotone alone, Classic, and Curve only.  The build option determines which set of tiles is used.  
Routering on a CNC machine is possible.
Reference 1:
https://en.wikipedia.org/wiki/Truchet_tiles
Reference 2, in French:
http://jean-paul.davalan.pagesperso-orange.fr/divers/truchet/truc.html

This script requires an initial source file, TruchetDuo22.3dm, which contains source tile geometry in certain formats, in quadrant 'III' of Top View.  The geometry may contain Curves, and/or, BRep surfaces. In order to improve the display quality by reducing pixel "jaggedness," if two BRep surfaces are in contact with each other, the darker of the two has a Curve of the same Tone or Color, along the shared border/(edge).  Edges along such a border are Hidden, because two adjoining edges of different Tone or Color disrupt the quality of the display.  (If a surface has no adjoining surface of a different Tone or Color, just the screen background, having non-hidden edges of the same Tone or Color would have about the same quality of display as having the Curve(s), but instead, all Edges are hidden after copying.)  The script also requires that each source tile have a unique name, shared for its BRep surface type and Curves.  This is done to facilitate looking up the tile objectlists, in MoI's geometry database.  
For Example:  For simplicity, the following description applies to the first tile set, which consists of 4 tiles, tile0, tile1, tile2, and tile3.  Each tile has two wide, black quarter circle arc surfaces, plus border curves of the same black color, separating 3 duotone surfaces which are of two colors or shades.  So tile0 is composed of ONE objectlist, contained in and filtered from, MoI's geometry database,containing 5 surfaces, and 4 curves.  Each of these 9 objects are named tile0.  Because Curve objects do not contain edges, in order to hide the Edges of the surfaces, Each of the 9 objects in the objectlist will be tested to see its type, and edges hidden for only the BReps, not the curves, or an error would result.
There are no other objects in the geometry database named tile0.  Edges are each governed by their respective surface, so the edges do not have to have names assigned.  Edges need to be re-hidden, after copy of the master source tile(s), to improve the screen image of the tiles.

The Grid, located in quadrant 'I' of Top View, is a manifestation of MoI's geometry database.  Display of a tile is equivalent to adding its objectlist to MoI's geometry database.  
Each gCell of the Grid will be occupied by a Tile.
The gCells of the Grid are arrainged in rows and columns.  A gCell, or a Tile,
is x = 10 units in width, and is indexed from left to right, and y = 10 units in height, indexed from bottom to top..

The lower left corner of a source Tile is the copy from point. The lower left corner of a gCell is the copy to point.  When the selected source Tile is copied to the Grid, the name of the source tile is unchanged, but the name of the copied tile (all of its surfaces and curves) is/are changed to gridCellName = ( col+ '_' + sourceTileIndex + '_' + row );.  This would permit the replacement (NOT Implemented) of a random tile with a tile of the proper parity, from sourceTiles array.
The parity4 of the index 0 tile of a set, modulus 4, is zero. (e.g. tile0,tile4)
The parity4 of the index 1 tile of a set, modulus 4, is one.  (e.g. tile1, tile5)
The parity4 of the index 2 tile of a set, modulus 4, is two.  (e.g. tile2, tile6)
The parity4 of the index 3 tile of a set, modulus 4, is three.(e.g. tile3, tile7)

Duotone Tiles are of two different parities.  A Tile has either parity "Light = 0," or parity "Dark = 1."  The color or tone of the surface at the lower left corner of a tile is its parity.  tile0 and tile2 are of Light parity, which is equivalent to being equal to 0, by modulus 2 remainder arithmetic.  tile1 and tile3 are of Dark parity.  The Tiles, gCells in the Grid, alternate in parity, which is a nonRandom situation.
The Tile in column,row position 0,0 is either of parity 0, or parity 1.

(The parity causes the "classic" tiles to not display all patterns, unless parity were to be coded differently.)

Operation of the script, for the first Build type:
Step1:
Load the TruchetDuo22.3dm file, which has the 5 sets of 4 source tiles.
(The user may hide the few edges showing, but this is not necessary.)  

Step2:
The array tileName[] is populated with the names of the 4 source tiles, in this case tile0, tile1, tile2, and tile3.  

Step3:
The 4 source tiles are selected by name, from MoI's Geometry database, and their objectlists are stored in array sourceTiles[].

Step4:
The parity of the gCell in quadrant 'I', ( 0, 0 ), var integer01, is randomly determined.

Step5:
The cells of the grid are populated by uniquely renamed pseudo-randomly chosen copies of the 4 source tile objectlists.  Parities do alternate, reducing "randomness."

Step 6:
if ChangeCheckbox is changed, either checked or unchecked, all the tiles in the grid are changed to a new pseudo-random pattern.

*/

function alert( msg )
{
	moi.ui.commandUI.alert( msg );
}

// From Michael:
function get_type( obj )
{
    switch(obj.type)
    {
    case 1: return 'CurveSegment';
    case 2: return 'Curve';
    case 3: return 'BRep';
    case 4: return 'Face';
    case 5: return 'ConstructionLine';
    case 6: return 'PointObject';
    case 7: return 'MeshObject';
    
    case 0:
    default: return 'Unknown';
    }
}

// Get a random integer between min and max, inclusive.
function getRandomIntInclusive(min, max)
{
	min = Math.ceil(min);
	max = Math.floor(max);
	return Math.floor(Math.random() * (max - min + 1)) + min; 
}

// Select the 2D objects which comprise one source tile, from the geometry
// database, by using the unique name of the objects.
// From MoI forum: 8259.2 In reply to 8259.1
// There should only be surfaces and curves in the .3dm source file.
// Other objects, such as solids, seem to work, but have not been tested much.
// Spheres were tried, and worked.
function selectTile( tileName )
{
	var objs = moi.geometryDatabase.getObjects();
	for ( var i = 0; i < objs.length; ++i )
	{
		var obj = objs.item(i);
		if ( obj.name == tileName )
			obj.selected = true;
	}
}

function copyTile( objectlist, basept, offsetpt )
{
	var factory = moi.command.createFactory( 'copy' );
	factory.setInput( 0, objectlist );	
	factory.setInput( 1, basept );
	factory.setInput( 2, offsetpt );	
	var copyV = factory.calculate();
	return copyV;
}

function Update( numRows, numColumns, tileSet, ChangeCheckbox )
{
	var vM = moi.vectorMath;
	var gd = moi.geometryDatabase;
	gd.deselectAll();
	var basePtyValue;
	var tileList = gd.createObjectList();	
	// Get the Names of 4 pre-created source tiles from Build tileSet.
	// Each source tile may be composed of multiple objects.
	var tileNames = new Array( 4 );
	if ( tileSet == 'tileSetA' )
	{	
		tileNames = [ 'tile0', 'tile1', 'tile2', 'tile3' ];
		basePtyValue = -11;
	}
	else if ( tileSet == 'tileSetB' )
	{	
		tileNames = [ 'tile4', 'tile5', 'tile6', 'tile7' ];
		basePtyValue = -22;	
	}
	else if ( tileSet == 'tileSetC' )
	{	
		tileNames = [ 'tile8', 'tile9', 'tile10', 'tile11' ];
		basePtyValue = -33;
	}	
	else if ( tileSet == 'tileSetD' )
	{	
		tileNames = [ 'tile12', 'tile13', 'tile14', 'tile15' ];	
		basePtyValue = -44;
	}	
	else if ( tileSet == 'tileSetE' )
	{	
		tileNames = [ 'tile16', 'tile17', 'tile18', 'tile19' ];	
		basePtyValue = -55;
	}
	
	// Get the 4 source Tiles as 4 objectLists in array sourceTiles.
	var sourceTiles = new Array();
	for ( var i = 0; i < tileNames.length; ++i )
	{
		selectTile( tileNames[i] );
		var objectlist = moi.geometryDatabase.getSelectedObjects();		
		sourceTiles.push( objectlist );		
		gd.deselectAll();		
	}

	// Basic setup code.
	var tileIndex; // Random Index to tile pattern # 0, 1, 2, or 3.
	var integer01; // Random 0 or 1;
	var parity;    // mod 2 remainder;
	var newtileIndex;
	var numRows = Math.abs(numRows);	
	var numColumns = Math.abs(numColumns);
	var basePtxValue;
	var basept; // Initially, lower left corner of tile0.
	var offsetpt; // Initially, the origin.
	var ni;
	var nj;
	var gridCellName;
	//var gridCellContents = new Array(); //NOT Implemented.
	var col,row;
	//var randomCol,randomRow; //NOT Implemented.
	//alert( ' numRows =  ' + numRows + '    numColumns =  ' + numColumns + '     change = ' + ChangeCheckbox )

	// Copy source tiles to fill the matrix and grid.
	// Each chosen source tile must have the correct parity, based upon
	// tile at 0,0, and the column,row being filled.

	tileIndex = getRandomIntInclusive(0, 3);	
	for ( var i = 0; i < numRows; ++i )
	{
		for ( var j = 0; j < numColumns; ++j)
		{
			integer01 = getRandomIntInclusive( 0, 1 ); 
			parity = ( tileIndex + i + j ) % 2; // modulus 2
			newtileIndex = 2 * integer01 + parity;
			basePtxValue = -44 + ( newtileIndex * 11 );
			//alert( ' tileIndex =  ' + tileIndex + '    basePtxValue =  ' + basePtxValue + '  newtileIndex = ' + newtileIndex );	
			basept = moi.vectorMath.createPoint( basePtxValue, basePtyValue, 0 );
			offsetpt = moi.vectorMath.createPoint( j * 10, i * 10, 0 );
	
			copyV = copyTile( sourceTiles[newtileIndex], basept, offsetpt );
	
			// Rename all of the objects in this Grid tile.
			// Note, edges are still unnamed, which should not matter.

			col = j.toString(); //column
			row = i.toString(); //row
			sourceTileIndex = newtileIndex.toString(); //('0','1','2','3').
			gridCellName = ( col+ '_' + sourceTileIndex + '_' + row );
			
// var coolVar = '123-abc-itchy-knee';
// var coolVarParts = coolVar.split('-'); // array of the items
// var1=coolVarParts[0]; //this will retrieve 123 
			// gridCellContents = gridCellName.split('_'); // Array of source indices. //NOT Implemented.
			
			for (k = 0; k < copyV.length; ++k) //Original good line.
			{
				copyV.item(k).name = gridCellName;

				// Hide edges of BRep objects.	
				var obj = copyV.item(k);
				var typeObj = get_type(obj);
				if (typeObj=='BRep')
				{
					copyV.item(k).getEdges().setProperty( 'hidden', true );
				}
				tileList.addObject( copyV.item(k));				
			}			
		}
	}
	return tileList;
}


/*
NOT IMPLEMENTED:
To change a single C tile, simply replace it with the tile patter C' = (C+2) mod.4 
 (n mod. 2 is the remainder in the division of n by 2, its value is 0 when n is even and 1 when n is odd. In C, java or javascript we would write n % 2).
*/ 
// Draft code: NOT Implemented.
// Get random i and random j
//	randomCol = getRandomIntInclusive( 0, numColumns );
//	randomRow = getRandomIntInclusive( 0, numRows );
	// Delete tile 'randomCol_sourceTileIndex_randomRow';
	// Replace with same parity.

function DoTile()
{
	var dlg = moi.ui.commandDialog;
	var ui = moi.ui.commandUI;
	var tileList = Update( ui.numRows.integerValue, ui.numColumns.integerValue, moi.ui.commandUI.Build.value, ui.ChangeCheckbox.value );
	moi.geometryDatabase.addObjects(tileList);

	while ( 1 )
	{
		if ( !dlg.waitForEvent() )
		{
			moi.geometryDatabase.removeObjects(tileList);			
			return;
		}
			
		if ( dlg.event == 'done' )
			break;
			
		if ( dlg.event == 'numRows' || dlg.event == 'numColumns' || dlg.event == 'Build' || dlg.event == 'ChangeCheckbox' )
		{
			moi.geometryDatabase.removeObjects(tileList);	
			var tileList = Update( ui.numRows.integerValue, ui.numColumns.integerValue, moi.ui.commandUI.Build.value, ui.ChangeCheckbox.value );
			moi.geometryDatabase.addObjects(tileList);
		}
	}
}
DoTile();
