function alert( msg )
{
	moi.ui.commandUI.alert( msg );
}

function sign(x) {
    return typeof x === 'number' ? x ? x < 0 ? -1 : 1 : x === x ? 0 : NaN : NaN;
}

//  Computes the R_F from Carlson symmetric form 
//  http://en.wikipedia.org/wiki/Carlson_symmetric_form#Series_Expansion
function RF( X, Y, Z )
{
	//var X, Y, Z;
	var result, A, lamda, dx, dy, dz;
	var MinError = 0.0000001;

	do
	{
       lamda = Math.sqrt(X * Y) + Math.sqrt(Y * Z) + Math.sqrt(Z * X);

       X = (X + lamda) / 4;
       Y = (Y + lamda) / 4;
       Z = (Z + lamda) / 4;

       A = (X + Y + Z) / 3;

       dx = (1 - X / A);
       dy = (1 - Y / A);
       dz = (1 - Z / A);
	}
    while (Math.max(Math.max(Math.abs(dx), Math.abs(dy)), Math.abs(dz)) > MinError);

    var E2, E3;
    E2 = dx * dy + dy * dz + dz * dx;
    E3 = dy * dx * dz;

    //http://dlmf.nist.gov/19.36#E1
    result = 1 - (1 / 10) * E2 + (1 / 14) * E3 + (1 / 24) * E2 * E2 - (3 / 44) * E2 * E3 - (5 / 208) * E2 * E2 * E2  + (3 / 104) * E3 * E3 + (1 / 16) * E2 * E2 * E3;

    result *= (1 / Math.sqrt(A));
    return result;
}

//  Computes the R_D from Carlson symmetric form
//  http://en.wikipedia.org/wiki/Carlson_symmetric_form#Series_Expansion
//  Constructed from R_J(x,y,z,z) which is equal to R_D(x,y,z)

function RD( X, Y, Z )
{
	//var X, Y, Z;
    var sum, fac, lamda, dx, dy, dz, A;
    var MinError = 0.000000001;

    sum = 0;
    fac = 1;

    do
{
		lamda = Math.sqrt(X * Y) + Math.sqrt(Y * Z) + Math.sqrt(Z * X);
        sum += fac / (Math.sqrt(Z) * (Z + lamda));

        fac /= 4;

        X = (X + lamda) / 4;
        Y = (Y + lamda) / 4;
        Z = (Z + lamda) / 4;

        A = (X + Y + 3 * Z) / 5;

        dx = (1 - X / A);
        dy = (1 - Y / A);
        dz = (1 - Z / A);
	}
    while (Math.max(Math.max(Math.abs(dx), Math.abs(dy)), Math.abs(dz)) > MinError);

    var E2, E3, E4, E5;
    E2 = dx * dy + dy * dz + 3 * dz * dz + 2 * dz * dx + dx * dz + 2 * dy * dz;
    E3 = dz * dz * dz + dx * dz * dz + 3 * dx * dy * dz + 2 * dy * dz * dz + dy * dz * dz + 2 * dx * dz * dz;
    E4 = dy * dz * dz * dz + dx * dz * dz * dz + dx * dy * dz * dz + 2 * dx * dy * dz * dz;
    E5 = dx * dy * dz * dz * dz;

    //http://dlmf.nist.gov/19.36#E2
    var result = (1 - (3 / 14) * E2 + (1 / 6) * E3 + (9 / 88) * E2 * E2 - (3 / 22) * E4 - (9 / 52) * E2 * E3 + (3 / 26) * E5 - (1 / 16) * E2 * E2 * E2 + (3 / 40) * E3 * E3 + (3 / 20) * E2 * E4 + (45 / 272) * E2 * E2 * E3 - (9 / 68) * (E3 * E4 + E2 * E5));

    result = 3.0 * sum + fac * result / (A * Math.sqrt(A));
    return result;
}

// Returns the incomplete elliptic integral of the first kind using Carlson symmetric forms. 
// u is the angle in radians, from 0 to PI/2 inclusive.
// Note that kk = k*k.   (m = kk)
function ei1F(snValue, kk)
{
	var u = snValue;
	var cosu2 = Math.cos(u);
	cosu2 *= cosu2;
	var sinu2 = Math.sin(u);
	sinu2 *= sinu2;	
	var result = Math.sin(u) * RF(cosu2, (1 - kk * sinu2), 1);
    return result;
}

// Returns the incomplete elliptic integral of the second kind using Carlson symmetric forms. 
// u is the angle in radians, from 0 to PI/2 inclusive.
// Note that kk = k*k.   (m = kk)
function ei2E(snValue, kk)
{ 
	var u = snValue;
	var result = Math.sin(u) * RF(Math.pow(Math.cos(u),2), 1 - kk * Math.pow(Math.sin(u),2), 1) - (1 / 3) * kk * Math.pow(Math.sin(u),3) * RD(Math.pow(Math.cos(u),2), 1 - kk * Math.pow(Math.sin(u),2), 1);
	return result;
}

function Update(hValue, eValue)
{
	var curvefactory = moi.command.createFactory( 'interpcurve' );
	
	var UI = moi.ui.commandUI;
	var numpoints = UI.numpoints.integerValue;
	var bottomcurve = UI.bot.value;
	var bot = 1.0;
	if ( bottomcurve )
	bot = -1.0;
	var slimcurve = UI.slim.value;
	var slim = 1.0;
	if ( slimcurve )
	slim = -1.0;	
	
	// Calculate values that do not vary with uValue (or uTilde).
	var k = Math.sqrt(0.5 - 3/(4*Math.sqrt(2 + eValue*eValue)));// page 6 of paper.
	var kk = k*k;
	var eValue = bot * eValue;	
	var eSign = sign(eValue);
	var eAbs = Math.abs(eValue); 
	var sqrt2eless1 = Math.sqrt(2*eAbs-1);
	var eSqrplus2 = eValue*eValue + 2;
	var sqrteSqrplus2 = Math.sqrt(eValue*eValue + 2);
	
	var tau = Math.sqrt((1 + eValue + sqrteSqrplus2) / (1 + eValue - sqrteSqrplus2));
	var hValue = bot * hValue;	
	var hSign = sign(hValue);
	var mu = 4 / (hValue * Math.pow(eSqrplus2,0.75));
	var xV1 = 1 + eAbs + sqrt2eless1;
	var PIo2 = Math.PI / 2;
	var m2 = 2 * ( Math.sqrt((1 + eAbs) * sqrt2eless1 )) / xV1;
	var m2m2 = m2 * m2;
	var Kofm2 =  ei1F( PIo2, m2m2 );
	var Kofk =  ei1F( PIo2, kk ); 
	var hsqr = hValue * hValue;
	var zetaV1 = 8 * Kofk / (mu * hsqr * eSqrplus2 * Kofm2);
	var zetaV2 = xV1 * xV1 / eSqrplus2;
	var zetaV3 = 2 * hSign * eValue * xV1 / eSqrplus2;

	var uMax = 2 * Kofk;//For rightmost bulge point.
// The maximum x occurs at the rightmost bulge point, where
// uTilde = 2 * Kofm2, and u = 2*Kofk.  (From page 7 formula)
// (Max z occurs at uTilde = 3*KofM2, where u = 4*Kofk, but last point will not plot.)
// Plot point for z == 0.  (tval of zero does not work).
// At z == 0, u == 0, so uTilde == K(m2), dn(uTilde,m2) = dn(K(m2),m2)
// Note: dn(K(m2),m2) = Math.sqrt(1-m2m2);
	var y = 0.0;
	var z = 0.0;	
	var dnKm2m2 = Math.sqrt(1-m2m2);
	var x0 = (-2 * eSign * xV1 * dnKm2m2) / (eSqrplus2 * hValue);
	var pt = moi.vectorMath.createPoint( x0, y, z );		
	curvefactory.createInput( 'point' );
	curvefactory.setInput( curvefactory.numInputs - 1, pt );	

	for ( var j = 1; j < numpoints; ++j )
	{
		var tval = j / (numpoints - 1);
		var uValue = slim * -tval * uMax;
		var uTilde = (( Kofm2/( 2 * Kofk )) * uValue ) + Kofm2;
		
		//	Get dnValue and amValue.
		var tol = 10.0e-11;
//		var snValue;
//		var cnValue;
		var dnValue;
		var t;
		/*Set up two arrays and AGM values*/
		var asav = [];
		var csav = [];
		asav[0] = 1.0;
		var b = Math.sqrt(1.0 - m2m2);
//		csav[0] = k; 
		csav[0] = m2;
		var n2 = 1.0;
		var i = 0;

		while( Math.abs( (csav[i]/asav[i]) ) > tol )
		{
			if( i > 10 )
			{
				/*mtherr( "ellpjf", OVERFLOW );*/
				break;
			}
			var a = asav[i];
			++i;
			csav[i] = 0.5 * ( a - b );
			var t = Math.sqrt( a * b );
			asav[i] = 0.5 * ( a + b );
			b = t;
			n2 += n2;
		}
		/* backward recurrence */
		// Find phi_subzero, starting at a[n] and c[n], and decrementing to n=0.
		// See paragraph 22.20.2 and .3 and .4 of nist website.
		var phi = n2 * asav[i] * uTilde;
		do
		{
			t = csav[i] * Math.sin(phi) / asav[i];
			b = phi;
			phi = 0.5 * (Math.asin(t) + phi);
		}
		while( --i );

//		snValue = Math.sin(phi);
		t = Math.cos(phi);
//		cnValue = t;
		dnValue = t/Math.cos(phi-b);
		amValue = phi; //amplitude

		var x = (-2 * eSign * xV1 * dnValue) / (eSqrplus2 * hValue); 
	
		var zeta_u = zetaV1 * (zetaV2 * ei2E(amValue,m2m2) + zetaV3 * amValue + ei1F(amValue,m2m2));

		// note, am(K(k),k) = PI/2, so am(K(m2),m2) = PI/2
		var zeta_0 = zetaV1 * (zetaV2 * ei2E(PIo2,m2m2) + zetaV3 * PIo2 + ei1F(PIo2,m2m2));		

		var z = zeta_u - zeta_0;

//		alert ("x =" + x + "\nz =" + z + "\namValue =" + amValue + "\ndnValue =" + dnValue + "\nzeta_u =" + zeta_u + "\nzeta_0 =" + zeta_0 + "\nmu =" + mu  );
		var pt = moi.vectorMath.createPoint( x, y, z );		
		curvefactory.createInput( 'point' );
		curvefactory.setInput( curvefactory.numInputs - 1, pt );
	}
	curvefactory.update();
	return curvefactory;
}

function DoVesicle()
{
	var hValue = moi.ui.commandUI.hValue.numericValue;
	var eValue = moi.ui.commandUI.eValue.numericValue;
	
// Note that "prefix" moi.ui.commandUI. is needed for code moved from .js to .htm file.
	var curvefactory = Update(hValue, eValue);
//	var curvefactory = moi.ui.commandUI.Update(uMax, hValue, eValue); //for code in .htm file.
	
	var dlg = moi.ui.commandDialog;
	
	while ( 1 )
	{
		if ( !dlg.waitForEvent() )
		{
			curvefactory.cancel();
			return false;
		}
			
		if ( dlg.event == 'done' )
			break;
			
		if ( dlg.event == 'hValue' || dlg.event == 'eValue' || dlg.event == 'numpoints' || dlg.event == 'Hslider'  || dlg.event == 'Eslider' || dlg.event == 'bot' || dlg.event == 'slim' )
		{
		
		var hValue = moi.ui.commandUI.hValue.numericValue;
		var eValue = moi.ui.commandUI.eValue.numericValue;
	
	if (hValue >= 0.0)
	{
		hValue = -0.01;
		alert ( " hValue must be < 0" + "\nhValue reset" ); 
	}

	if (eValue <= 0.5)
		{
			eValue = 0.501;
			alert ( " eValue must be > 0.5" + "\neValue reset" ); 
		}		
		
			curvefactory.cancel();
			var curvefactory = Update(hValue, eValue); //for code in .js
			//curvefactory = moi.ui.commandUI.Update(hValue, eValue); // for code in .htm
			moi.ui.commandUI.updating = false;			
		}
	}
	curvefactory.commit();
}
DoVesicle();







